"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.autoHeal = void 0;
const codeceptjs_1 = require("codeceptjs");
const AutoHeal_1 = require("../ElementHealer/AutoHeal");
const GetCurrentElementInfo_1 = require("../ElementHealer/GetCurrentElementInfo");
const GetHealedElementInfo_1 = require("../ElementHealer/GetHealedElementInfo");
const GlobalVariable_1 = require("../GlobalVariable");
const util = require('util');
const tryTo = codeceptjs.container.plugins('tryTo');
const { I } = inject();
class autoHeal extends codeceptjs_1.Helper {
    /**
    * Scrolls element into viewport, waits for that element to become visible & clickable, then performs click action.
    * Attempts auto healing of the locator
    * The second parameter is a context (CSS or XPath locator) to narrow the search.
    * ```js
    * // simple link
    * z.click('Logout');
    * // button of form
    * z.click('Submit');
    * // CSS button
    * z.click('#form input[type=submit]');
    * // XPath
    * z.click('//form/*[@type=submit]');
    * // link in context
    * z.click('Logout', '#nav');
    * // using strict locator
    * z.click({css: 'nav a.login'});
    * ```
    * @param {string} locator CSS|XPath|strict locator.
    * @param {string} context (optional, null by default) element to search in CSS|XPath|Strict locator.
    */
    static click(locator, locatorName, timeout = 120, retryCount = 0, context) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITAITED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.click(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in AUTOHEAL process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                console.log('-----Step Passed-----');
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.waitForClickable(locator);
                                if (util.isNullOrUndefined(context)) {
                                    yield I.click(locator);
                                }
                                else {
                                    yield I.click(locator, context);
                                }
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.waitForClickable(locator);
                                if (util.isNullOrUndefined(context)) {
                                    yield I.click(locator);
                                }
                                else {
                                    yield I.click(locator, context);
                                }
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
   * Waits for element to be visible, Clears element value & fills given value.
  * Attempts auto healing of the locator
   * ```js
   * // by label
   * z.fillField('Email', 'hello@world.com');
   * // by name
   * z.fillField('password', secret('123456'));
   * // by CSS
   * z.fillField('form#login input[name=username]', 'John');
   * // or by strict locator
   * z.fillField({css: 'form#login input[name=username]'} 'John');
   * ```
   * @param {string} locator CSS|XPath|strict locator.
   * @param {string} value
   */
    static fillField(locator, value, locatorName, timeout = 120, retryCount = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.fillField(GlobalVariable_1.GlobalVariable.HealedElement, value, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in AUTOHEAL process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.clearField(locator);
                                yield I.fillField(locator, value);
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                yield I.clearField(locator);
                                yield I.fillField(locator, value);
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
    * Scrolls element into viewport, waits for that element to become visible & enabled, then clears field value.
   * Attempts auto healing of the locator
    * ```js
    * z.clearField('Email');
    * z.clearField('user[email]');
    * z.clearField('#email');
    * ```
    * @param {string} locator CSS|XPath|strict locator.
    */
    static clearField(locator, locatorName, timeout = 120, retryCount = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.clearField(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.clearField(locator);
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                yield I.clearField(locator);
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
    * Checks the visibilty of an element.
    * Throws error if any of the action fails in two attempts.
    * ```js
    * z.seeElement('#modal');
    * ```
    * @param {string} locator CSS|XPath|strict locator.
    */
    static seeElement(locator, locatorName, timeout = 120, retryCount = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForVisible(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.seeElement(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.seeElement(locator);
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                yield I.seeElement(locator);
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
  * Waits for an element to be visible.
  * Element can be located by CSS or XPath.
  * ```js
  * z.attachFile('Avatar', 'data/avatar.jpg');
  * z.attachFile('form input[name=avatar]', 'data/avatar.jpg');
  * ```
  * @param {string} locator label|name|CSS|XPath|strict locator.
  * @param {string} filepath local file path relative to codecept.json config file
  */
    static attachFile(locator, filepath, locatorName, timeout = 120, retryCount = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.attachFile(GlobalVariable_1.GlobalVariable.HealedElement, filepath, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.attachFile(locator, filepath);
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                yield I.attachFile(locator, filepath);
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
    * Scroll element into viewport.
    * ```js
    * z.scrollIntoView('#submit');
    * z.scrollIntoView('#submit', { behavior: "smooth", block: "center", inline: "center" });
    * z.scrollIntoView('#submit', true);
    * ```
    * @param {string} locator CSS|XPath|strict locator.
    * @param {any} alignTop optional, or scrollIntoViewOptions (optional),
    * see https://developer.mozilla.org/en-US/docs/Web/API/Element/scrollIntoView.
    
    */
    static scrollIntoView(locator, alignTop) {
        return __awaiter(this, void 0, void 0, function* () {
            if (alignTop == null || alignTop == undefined) {
                yield I.scrollIntoView(locator, alignTop);
            }
            else {
                yield I.scrollIntoView(locator, { block: "start" });
            }
        });
    }
    /**
    * Scrolls element into viewport, waits for that element to become visible & clickable, then performs double click action.
    * Throws error if any of the action fails in two attempts.
    * The second parameter is a context (CSS or XPath locator) to narrow the search.
    * ```js
    * z.doubleClick('Edit');
    * z.doubleClick('Edit', '.actions');
    * z.doubleClick({css: 'button.accept'});
    * z.doubleClick('.btn.edit');
    * ```
    * @param {string} locator CSS|XPath|strict locator.
    * @param {string} context (optional, null by default) element to search in CSS|XPath|Strict locator.
    */
    static doubleClick(locator, locatorName, timeout = 120, retryCount = 0, context) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.doubleClick(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                console.log('-----Step Passed-----');
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.waitForClickable(locator);
                                if (util.isNullOrUndefined(context)) {
                                    yield I.doubleClick(locator);
                                }
                                else {
                                    yield I.doubleClick(locator, context);
                                }
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.waitForClickable(locator);
                                if (util.isNullOrUndefined(context)) {
                                    yield I.doubleClick(locator);
                                }
                                else {
                                    yield I.doubleClick(locator, context);
                                }
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
        * Scrolls element into viewport, waits for that element to become visible & enabled, then Force clicks an element
        
        * ```js
        * z.forceClick('#hiddenButton');
        * z.forceClick('Click me', '#hidden');
        * ```
       * @param {string} locator CSS|XPath|strict locator.
        * @param {string} context (optional, null by default) element to search in CSS|XPath|Strict locator.
        */
    static forceClick(locator, locatorName, timeout = 120, retryCount = 0, context) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.forceClick(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.waitForEnabled(locator);
                                if (util.isNullOrUndefined(context)) {
                                    yield I.forceClick(locator);
                                }
                                else {
                                    yield I.forceClick(locator, context);
                                }
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.waitForEnabled(locator);
                                if (util.isNullOrUndefined(context)) {
                                    yield I.forceClick(locator);
                                }
                                else {
                                    yield I.forceClick(locator, context);
                                }
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
     * Scrolls element into viewport, waits for that element to become visible & clickable, then Drag an item to a destination element.
     *
     * ```js
     * z.dragAndDrop('#dragHandle', '#container');
     * ```
     * @param {string} srcLocator CSS|XPath|strict locator.
     * @param {string} destLocator CSS|XPath|Strict locator.
     */
    static dragAndDrop(srcLocator, destLocator) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (srcLocator.startsWith("/") || srcLocator.startsWith(".") || srcLocator.startsWith("#") || srcLocator.startsWith("{")) {
                    yield I.waitForElement(srcLocator);
                    yield I.scrollIntoView(srcLocator, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForVisible(srcLocator);
                    yield I.waitForClickable(srcLocator);
                    yield I.waitForVisible(destLocator);
                    yield I.dragAndDrop(srcLocator, destLocator);
                }
                else {
                    yield I.dragAndDrop(srcLocator, destLocator);
                }
            }
            catch (error) {
                throw new Error("dragAndDrop : Error while performing drag and drop from element located by : " + srcLocator + " to " + destLocator);
            }
        });
    }
    /**
   * Scrolls element into viewport, waits for that element to become visible & clickable, then selects an option in a drop-down select.
   * Attempts auto healing on failure
   *
   * ```js
   * z.selectOption('Choose Plan', 'Monthly'); // select by label
   * z.selectOption('subscription', 'Monthly'); // match option by text
   * z.selectOption('subscription', '0'); // or by value
   * z.selectOption('//form/select[@name=account]','Premium');
   * z.selectOption('form select[name=account]', 'Premium');
   * z.selectOption({css: 'form select[name=account]'} 'Premium');
   * ```
   * Provide an array for the second argument to select multiple options.
   * ```js
   * z.selectOption('Which OS do you use?', ['Android', 'iOS']);
   * ```
   * @param {string} locator CSS|XPath|strict locator.
   * @param {string|string[]} option CSS|XPath|Strict locator.
   */
    static selectOption(locator, option, locatorName, timeout = 120, retryCount = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.selectOption(GlobalVariable_1.GlobalVariable.HealedElement, option, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.waitForVisible(locator);
                                yield I.waitForClickable(locator);
                                yield I.selectOption(locator, option);
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.waitForVisible(locator);
                                yield I.waitForClickable(locator);
                                yield I.selectOption(locator, option);
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
      * Scrolls element into viewport, waits for that element to become visible & clickable. Also checks if element is already checked.
      * Then selects a checkbox or radio button.
      *
      * The second parameter is a context (CSS or XPath locator) to narrow the search.
      * ```js
      * z.checkOption('#agree');
      * z.checkOption('I Agree to Terms and Conditions');
      * z.checkOption('agree', '//form');
      * ```
      * @param {string} locator checkbox located by label | name | CSS | XPath | strict locator.
      * @param {string} context optional element located by CSS | XPath | strict locator.
      */
    static checkOption(locator, locatorName, timeout = 120, retryCount = 0, context) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.checkOption(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                if (!util.isNullOrUndefined(context)) {
                                    yield I.checkOption(locator, context);
                                }
                                else {
                                    yield I.checkOption(locator);
                                }
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                if (!util.isNullOrUndefined(context)) {
                                    yield I.checkOption(locator, context);
                                }
                                else {
                                    yield I.checkOption(locator);
                                }
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
 * Scrolls element into viewport, waits for that element to become visible.
 * Then Checks that the given input field or textarea equals to given value.
 *
 * ```js
 * z.seeInField('Username', 'davert');
 * z.seeInField({css: 'form textarea'},'Type your comment here');
 * z.seeInField('#searchform input','Search');
 * ```
 * @param {string} field located by label|name|CSS|XPath|strict locator.
 * @param {string} value value to check.
 */
    static seeInField(locator, value, locatorName, timeout = 120, retryCount = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.seeInField(GlobalVariable_1.GlobalVariable.HealedElement, value, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.waitForVisible(locator);
                                yield I.seeInField(locator, value);
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.waitForVisible(locator);
                                yield I.seeInField(locator, value);
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
   * Checks for element to be present, scrolls into view & Moves cursor to element matched by locator.
   * Extra shift can be set with offsetX and offsetY options.
   *
   * ```js
   * z.moveCursorTo('.tooltip');
   * z.moveCursorTo('#submit', 5,5);
   * ```
   *
   * @param {CodeceptJS.LocatorOrString} locator located by CSS|XPath|strict locator.
   * @param {number} [offsetX=0] (optional, `0` by default) X-axis offset.
   * @param {number} [offsetY=0] (optional, `0` by default) Y-axis offset.
   *
   *
   */
    static moveCursorTo(locator, locatorName, offsetX, offsetY, timeout = 120, retryCount = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            if (util.isNullOrUndefined(offsetX) && util.isNullOrUndefined(offsetY)) {
                                                yield this.moveCursorTo(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                            }
                                            else if (util.isNullOrUndefined(offsetX) && !(util.isNullOrUndefined(offsetY))) {
                                                yield this.moveCursorTo(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, 0, offsetY, timeout, retryCount);
                                            }
                                            else if (!(util.isNullOrUndefined(offsetX)) && util.isNullOrUndefined(offsetY)) {
                                                yield this.moveCursorTo(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, offsetX, 0, timeout, retryCount);
                                            }
                                            else {
                                                yield this.moveCursorTo(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, offsetX, offsetY, timeout, retryCount);
                                            }
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                if (util.isNullOrUndefined(offsetX) && util.isNullOrUndefined(offsetY)) {
                                    yield I.moveCursorTo(locator);
                                }
                                else if (util.isNullOrUndefined(offsetX) && !(util.isNullOrUndefined(offsetY))) {
                                    yield I.moveCursorTo(locator, 0, offsetY);
                                }
                                else if (!(util.isNullOrUndefined(offsetX)) && util.isNullOrUndefined(offsetY)) {
                                    yield I.moveCursorTo(locator, offsetX, 0);
                                }
                                else {
                                    yield I.moveCursorTo(locator, offsetX, offsetY);
                                }
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                if (util.isNullOrUndefined(offsetX) && util.isNullOrUndefined(offsetY)) {
                                    yield I.moveCursorTo(locator);
                                }
                                else if (util.isNullOrUndefined(offsetX) && !(util.isNullOrUndefined(offsetY))) {
                                    yield I.moveCursorTo(locator, 0, offsetY);
                                }
                                else if (!(util.isNullOrUndefined(offsetX)) && util.isNullOrUndefined(offsetY)) {
                                    yield I.moveCursorTo(locator, offsetX, 0);
                                }
                                else {
                                    yield I.moveCursorTo(locator, offsetX, offsetY);
                                }
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
    * Performs a swipe Right inside an element.
    * Can be slow or fast swipe.
    * ```js
    * z.swipeRight('#container');
    *
    * @param {string} locator an element on which to perform swipe
    * @param {string}[speed='slow'] a speed to perform: `slow` or `fast`.
    */
    static swipeRight(locator, speed, locatorName, timeout = 120, retryCount = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.swipeRight(GlobalVariable_1.GlobalVariable.HealedElement, speed, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.swipeRight(locator, speed);
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                yield I.swipeRight(locator, speed);
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
* Waits for the visibility of element & for the specified value to be in value attribute.
* ```js
* z.waitForValue('//input', "GoodValue");
* ```
* @param {string} locator input field locator.
* @param {string} value
* @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
*/
    static waitForValue(locator, value, locatorName, timeout = 120, retryCount = 0, context) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForValue(locator, value);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.waitForValue(GlobalVariable_1.GlobalVariable.HealedElement, value, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
* Waits for the visibility of element.
* ```js
* z.waitForVisible('#popup');
* ```
* @param {string} locator CSS|XPath|strict locator.
* @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
*/
    static waitForVisible(locator, locatorName, timeout = 120, retryCount = 0, context) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForVisible(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.waitForVisible(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
   * Waits for an element to be removed or become invisible on a page.
   * Element can be located by CSS or XPath.
   * ```js
   * z.waitForInvisible('#popup');
   * ```
   * @param {string} locator CSS|XPath|strict locator.
   * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
   */
    static waitForInvisible(locator, locatorName, timeout = 120, retryCount = 0, context) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForInvisible(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.waitForInvisible(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
    * Waits for an element to become enabled.
    * Element can be located by CSS or XPath.
    * ```js
    * z.waitForEnabled('#popup');
    * ```
    * @param {string} locator CSS|XPath|strict locator.
    * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
    */
    static waitForEnabled(locator, locatorName, timeout = 120, retryCount = 0, context) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForEnabled(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.waitForEnabled(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
     * Waits for an element to be present on page.
     * Element can be located by CSS or XPath.
     * ```js
     * z.waitForElement('.btn.continue');
     * z.waitForElement('.btn.continue', 5); // wait for 5 secs
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
     */
    static waitForElement(locator, locatorName, timeout = 120, retryCount = 0, context) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.waitForElement(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
   * Waits for an element to be clickable.
   * Element can be located by CSS or XPath.
   * ```js
   * z.waitForClickable('.btn.continue');
   * z.waitForClickable('.btn.continue', 5); // wait for 5 secs
   * ```
   * @param {string} locator CSS|XPath|strict locator.
   * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
   */
    static waitForClickable(locator, locatorName, timeout = 120, retryCount = 0, context) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForClickable(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.waitForClickable(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    static grabTextFrom(locator, locatorName, timeout = 120, retryCount = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.grabTextFrom(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                            resolve(GlobalVariable_1.GlobalVariable.text);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                GlobalVariable_1.GlobalVariable.text = yield I.grabTextFrom(locator);
                                if (GlobalVariable_1.GlobalVariable.text != undefined) {
                                    if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                        if (retryCount == 2)
                                            yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                        else
                                            GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                        yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                    }
                                    yield I.wait(10);
                                    resolve(GlobalVariable_1.GlobalVariable.text);
                                }
                                else
                                    reject('Cannot grab text from the locator even after healing: ' + locator);
                            }
                        }
                        else {
                            if (result) {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                GlobalVariable_1.GlobalVariable.text = yield I.grabTextFrom(locator);
                                resolve(GlobalVariable_1.GlobalVariable.text);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    static grabTextFromAll(locator, locatorName, timeout = 120, retryCount = 0, context) {
        return __awaiter(this, void 0, void 0, function* () {
            let text = [];
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.grabTextFromAll(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                            resolve(GlobalVariable_1.GlobalVariable.textFromAll);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                GlobalVariable_1.GlobalVariable.textFromAll = yield I.grabTextFromAll(locator);
                                if (GlobalVariable_1.GlobalVariable.textFromAll.length != 0) {
                                    if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                        if (retryCount == 2)
                                            yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                        else
                                            GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                        yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                    }
                                    yield I.wait(10);
                                    resolve(GlobalVariable_1.GlobalVariable.textFromAll);
                                }
                                else
                                    reject('Cannot grab text from the locator even after healing: ' + locator);
                            }
                        }
                        else {
                            if (result) {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                GlobalVariable_1.GlobalVariable.textFromAll = yield I.grabTextFromAll(locator);
                                resolve(GlobalVariable_1.GlobalVariable.textFromAll);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
        * Retrieves an attribute from an element located by CSS or XPath and returns it to test. An array as a result will be returned if there are more than one matched element.
        * Resumes test execution, so should be used inside static async with await operator.
        * ```js
        * let hint = await z.grabAttributeFrom('#tooltip', 'title');
        * ```
        * @param {string} locator element located by CSS|XPath|strict locator.
        * @param {string} attribute attribute name.
        * @returns {string} text
        */
    static grabAttributeFrom(locator, attribute, locatorName, elementLocatorStrategy = 'xpath', timeout = 120, retryCount = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            GlobalVariable_1.GlobalVariable.AttributeValue = '';
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                GlobalVariable_1.GlobalVariable.text = '';
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.grabAttributeFrom(GlobalVariable_1.GlobalVariable.HealedElement, attribute, locatorName, elementLocatorStrategy, timeout, retryCount);
                                            resolve(GlobalVariable_1.GlobalVariable.AttributeValue);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.waitForVisible(locator);
                                GlobalVariable_1.GlobalVariable.AttributeValue = yield this.grabPropertyOrAttributeValue(locator, attribute, elementLocatorStrategy);
                                if (GlobalVariable_1.GlobalVariable.AttributeValue != undefined) {
                                    if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                        if (retryCount == 2)
                                            yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                        else
                                            GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                        yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                    }
                                    yield I.wait(10);
                                    resolve(GlobalVariable_1.GlobalVariable.AttributeValue);
                                }
                                else
                                    reject('Cannot grab text from the locator even after healing: ' + locator);
                            }
                        }
                        else {
                            if (result) {
                                yield I.waitForVisible(locator);
                                GlobalVariable_1.GlobalVariable.AttributeValue = yield this.grabPropertyOrAttributeValue(locator, attribute, elementLocatorStrategy);
                                resolve(GlobalVariable_1.GlobalVariable.AttributeValue);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    static grabAttributeFromAll(locator, attribute, locatorName, elementLocatorStrategy = 'xpath', timeout = 120, retryCount = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            GlobalVariable_1.GlobalVariable.AttributeValueFromAll = [];
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                GlobalVariable_1.GlobalVariable.text = '';
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.grabAttributeFromAll(GlobalVariable_1.GlobalVariable.HealedElement, attribute, locatorName, elementLocatorStrategy, timeout, retryCount);
                                            resolve(GlobalVariable_1.GlobalVariable.AttributeValueFromAll);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.waitForVisible(locator);
                                GlobalVariable_1.GlobalVariable.AttributeValueFromAll = yield this.grabPropertyOrAttributeValueFromAll(locator, attribute, elementLocatorStrategy);
                                if (GlobalVariable_1.GlobalVariable.AttributeValueFromAll.length != 0) {
                                    if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                        if (retryCount == 2)
                                            yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                        else
                                            GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                        yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                    }
                                    yield I.wait(10);
                                    resolve(GlobalVariable_1.GlobalVariable.AttributeValueFromAll);
                                }
                                else
                                    reject('Cannot grab text from the locator even after healing: ' + locator);
                            }
                        }
                        else {
                            if (result) {
                                yield I.waitForVisible(locator);
                                GlobalVariable_1.GlobalVariable.AttributeValueFromAll = yield this.grabPropertyOrAttributeValueFromAll(locator, attribute, elementLocatorStrategy);
                                resolve(GlobalVariable_1.GlobalVariable.AttributeValueFromAll);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
         * Checks for element to be present, scrolls into view, waits for element to be visible and retrieves a value from a form element located by CSS or XPath, returns it to test.
         * Resumes test execution, so **should be used inside async function with `await`** operator.
         *
         * ```js
         * let email = await z.grabValueFrom('input[name=email]');
         * ```
         * @param {CodeceptJS.LocatorOrString} locator field located by label|name|CSS|XPath|strict locator.
         * @returns {Promise<any>} attribute value
         *
         */
    static grabValueFrom(locator, locatorName, elementLocatorStrategy = 'xpath', timeout = 120, retryCount = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            GlobalVariable_1.GlobalVariable.AttributeValue = '';
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                GlobalVariable_1.GlobalVariable.text = '';
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.grabValueFrom(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, elementLocatorStrategy, timeout, retryCount);
                                            resolve(GlobalVariable_1.GlobalVariable.AttributeValue);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.waitForVisible(locator);
                                GlobalVariable_1.GlobalVariable.AttributeValue = yield this.grabPropertyOrAttributeValue(locator, 'value', elementLocatorStrategy);
                                if (GlobalVariable_1.GlobalVariable.AttributeValue != undefined) {
                                    if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                        if (retryCount == 2)
                                            yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                        else
                                            GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                        yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                    }
                                    yield I.wait(10);
                                    resolve(GlobalVariable_1.GlobalVariable.AttributeValue);
                                }
                                else
                                    reject('Cannot grab value from the locator even after healing: ' + locator);
                            }
                        }
                        else {
                            if (result) {
                                yield I.waitForVisible(locator);
                                GlobalVariable_1.GlobalVariable.AttributeValue = yield this.grabPropertyOrAttributeValue(locator, 'value', elementLocatorStrategy);
                                resolve(GlobalVariable_1.GlobalVariable.AttributeValue);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    static grabValueFromAll(locator, locatorName, elementLocatorStrategy = 'xpath', timeout = 120, retryCount = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            GlobalVariable_1.GlobalVariable.AttributeValueFromAll = [];
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                GlobalVariable_1.GlobalVariable.text = '';
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.grabValueFromAll(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, elementLocatorStrategy, timeout, retryCount);
                                            resolve(GlobalVariable_1.GlobalVariable.AttributeValueFromAll);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.waitForVisible(locator);
                                GlobalVariable_1.GlobalVariable.AttributeValueFromAll = yield this.grabPropertyOrAttributeValueFromAll(locator, 'value', elementLocatorStrategy);
                                if (GlobalVariable_1.GlobalVariable.AttributeValueFromAll.length != 0) {
                                    if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                        if (retryCount == 2)
                                            yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                        else
                                            GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                        yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                    }
                                    yield I.wait(10);
                                    resolve(GlobalVariable_1.GlobalVariable.AttributeValueFromAll);
                                }
                                else
                                    reject('Cannot grab text from the locator even after healing: ' + locator);
                            }
                        }
                        else {
                            if (result) {
                                yield I.waitForVisible(locator);
                                GlobalVariable_1.GlobalVariable.AttributeValueFromAll = yield this.grabPropertyOrAttributeValueFromAll(locator, 'value', elementLocatorStrategy);
                                resolve(GlobalVariable_1.GlobalVariable.AttributeValueFromAll);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    static grabPropertyOrAttributeValue(locator, attribute, elementLocatorStrategy = 'xpath') {
        return __awaiter(this, void 0, void 0, function* () {
            let value = '';
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const browser = yield this.prototype.helpers['WebDriver'].browser;
                    const elementRef = yield browser.findElement(elementLocatorStrategy, locator);
                    Object.keys(elementRef)
                        .forEach(function eachKey(key) {
                        return __awaiter(this, void 0, void 0, function* () {
                            value = yield browser.getElementAttribute(elementRef[key], attribute);
                            if (value == null) {
                                value = yield browser.getElementProperty(elementRef[key], attribute);
                            }
                            resolve(value);
                        });
                    });
                }
                catch (error) {
                    throw new Error('Error: Grab Property and attribute is throwing error for locator : ' + locator + 'while fetching attribute or property : ' + attribute);
                }
            }));
        });
    }
    static grabPropertyOrAttributeValueFromAll(locator, attribute, elementLocatorStrategy = 'xpath') {
        return __awaiter(this, void 0, void 0, function* () {
            let value;
            let valueArray = [];
            let count = 0;
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const browser = yield this.prototype.helpers['WebDriver'].browser;
                    const elementRef = yield browser.findElements(elementLocatorStrategy, locator);
                    for (const element in elementRef) {
                        Object.keys(elementRef[element])
                            .forEach(function eachKey(key) {
                            return __awaiter(this, void 0, void 0, function* () {
                                value = yield browser.getElementAttribute(elementRef[element][key], attribute);
                                if (value == null) {
                                    value = yield browser.getElementProperty(elementRef[element][key], attribute);
                                }
                                count = count + 1;
                                valueArray.push(value);
                                if (count == elementRef.length) {
                                    resolve(valueArray);
                                }
                            });
                        });
                    }
                }
                catch (error) {
                    reject(error);
                    throw new Error('Error: Grab attribute from all is throwing error for locator : ' + locator + 'while fetching attribute or property : ' + attribute);
                }
            }));
        });
    }
    /**
         * Scrolls element into viewport, waits for that element to become visible.
         * Then Verifies that the specified checkbox is not checked.
         *
         * ```js
         * z.dontSeeCheckboxIsChecked('#agree');
         * z.dontSeeCheckboxIsChecked('I agree to terms');
         * z.dontSeeCheckboxIsChecked('agree');
         * ```
         * @param {string} field  located by label|name|CSS|XPath|strict locator.
         */
    static dontSeeCheckboxIsChecked(locator, locatorName, timeout = 120, retryCount = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.dontSeeCheckboxIsChecked(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.dontSeeCheckboxIsChecked(locator);
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.dontSeeCheckboxIsChecked(locator);
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
         * Scrolls element into viewport, waits for that element to become visible.
         * Then Verifies that the specified checkbox is checked.
         *
         * ```js
         * z.seeCheckboxIsChecked('Agree');
         * z.seeCheckboxIsChecked('#agree');
         * z.seeCheckboxIsChecked({css: '#signup_form input[type=checkbox]'});
         * ```
         * @param {string} field  located by label|name|CSS|XPath|strict locator.
         */
    static seeCheckboxIsChecked(locator, locatorName, timeout = 120, retryCount = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.seeCheckboxIsChecked(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.seeCheckboxIsChecked(locator);
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.seeCheckboxIsChecked(locator);
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
         * Checks that all elements with given locator have given attributes.
         *
         * ```js
         * z.seeAttributesOnElements('//form', { method: "post"});
         * ```
         *
         * @param {CodeceptJS.LocatorOrString} locator located by CSS|XPath|strict locator.
         * @param {object} attributes attributes and their values to check.
         *
         */
    static seeAttributesOnElements(locator, attributes, locatorName, timeout = 120, retryCount = 0, context) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.seeAttributesOnElements(GlobalVariable_1.GlobalVariable.HealedElement, attributes, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.seeAttributesOnElements(locator, attributes);
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                yield I.seeAttributesOnElements(locator, attributes);
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
        * Appends text into the field.
        * A field can be located by text, accessibility id, id.
        *
        * ```js
        * z.appendField('name', 'davert');
        * ```
        *
        * @param {string} field
        * @param {string} value
        */
    static appendField(locator, value, locatorName, timeout = 120, retryCount = 0, context) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.appendField(GlobalVariable_1.GlobalVariable.HealedElement, value, locatorName, timeout, retryCount);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.click(locator);
                                yield I.appendField(locator, value);
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(true);
                            }
                        }
                        else {
                            if (result) {
                                yield I.click(locator);
                                yield I.appendField(locator, value);
                                resolve(true);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
             * Retrieves the innerHTML from an element located by CSS or XPath and returns it to test.
             * Resumes test execution, so **should be used inside async function with `await`** operator.
             * If more than one element is found - an array of HTMLs returned.
             *
             * ```js
             * let postHTML = await z.grabHTMLFrom('#post');
             * ```
             *
             * @param {CodeceptJS.LocatorOrString} element located by CSS|XPath|strict locator.
             * @returns {Promise<string>} HTML code for an element
             */
    static grabHTMLFrom(locator, locatorName, timeout = 120, retryCount = 0, context) {
        return __awaiter(this, void 0, void 0, function* () {
            GlobalVariable_1.GlobalVariable.postHTML = '';
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.grabHTMLFrom(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                            resolve(GlobalVariable_1.GlobalVariable.postHTML);
                                            resolve(true);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.waitForVisible(locator);
                                GlobalVariable_1.GlobalVariable.postHTML = yield I.grabHTMLFrom(locator);
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(GlobalVariable_1.GlobalVariable.postHTML);
                            }
                        }
                        else {
                            if (result) {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.waitForVisible(locator);
                                resolve(GlobalVariable_1.GlobalVariable.postHTML);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
             * Retrieves all the innerHTML from elements located by CSS or XPath and returns it to test.
             * Resumes test execution, so should be used inside async function with await operator.
             *
             * ```js
             * let postHTML = await z.grabHTMLFromAll('#post');
             * ```
             *
             * @param {CodeceptJS.LocatorOrString} element located by CSS|XPath|strict locator.
             * @returns {Promise<string>} HTML code for an element
             */
    static grabHTMLFromAll(locator, locatorName, timeout = 120, retryCount = 0, context) {
        return __awaiter(this, void 0, void 0, function* () {
            GlobalVariable_1.GlobalVariable.postHTMLFromAll = '';
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.grabHTMLFromAll(GlobalVariable_1.GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                            resolve(GlobalVariable_1.GlobalVariable.postHTMLFromAll);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.waitForVisible(locator);
                                GlobalVariable_1.GlobalVariable.postHTMLFromAll = yield I.grabHTMLFromAll(locator);
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(GlobalVariable_1.GlobalVariable.postHTMLFromAll);
                            }
                        }
                        else {
                            if (result) {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.waitForVisible(locator);
                                GlobalVariable_1.GlobalVariable.postHTMLFromAll = yield I.grabHTMLFromAll(locator);
                                resolve(GlobalVariable_1.GlobalVariable.postHTMLFromAll);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    /**
            * Retrieves the cssProperty from an element located by CSS or XPath and returns it to test.
            * Resumes test execution, so **should be used inside async function with `await`** operator.
            * If more than one element is found - an array of HTMLs returned.
            *
            * ```js
            * const value = await I.grabCssPropertyFrom('h3', 'font-weight');
            * ```
            *
            * @param {CodeceptJS.LocatorOrString} element located by CSS|XPath|strict locator.
            *  @param {CodeceptJS.String} cssProperty CSS property name.
            * @returns {Promise<string>} CSS value
            */
    static grabCssPropertyFrom(locator, cssProperty, locatorName, timeout = 120, retryCount = 0, context) {
        return __awaiter(this, void 0, void 0, function* () {
            GlobalVariable_1.GlobalVariable.postHTML = '';
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.grabCssPropertyFrom(GlobalVariable_1.GlobalVariable.HealedElement, cssProperty, locatorName, timeout, retryCount);
                                            resolve(GlobalVariable_1.GlobalVariable.postHTML);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.waitForVisible(locator);
                                GlobalVariable_1.GlobalVariable.postHTML = yield I.grabCssPropertyFrom(locator, cssProperty);
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(GlobalVariable_1.GlobalVariable.postHTML);
                            }
                        }
                        else {
                            if (result) {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.waitForVisible(locator);
                                GlobalVariable_1.GlobalVariable.postHTML = yield I.grabCssPropertyFrom(locator, cssProperty);
                                resolve(GlobalVariable_1.GlobalVariable.postHTML);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
    static grabCssPropertyFromAll(locator, cssProperty, locatorName, timeout = 120, retryCount = 0, context) {
        return __awaiter(this, void 0, void 0, function* () {
            GlobalVariable_1.GlobalVariable.postHTMLFromAll = '';
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (retryCount == 0)
                    GlobalVariable_1.GlobalVariable.OriginalXpath = locator;
                if (retryCount <= 1) {
                    yield tryTo(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            yield I.waitForElement(locator, timeout);
                            GlobalVariable_1.GlobalVariable.error = 'NA';
                        }
                        catch (error) {
                            console.log('Error with element : ' + error);
                            GlobalVariable_1.GlobalVariable.error = error;
                        }
                    })).then((result) => __awaiter(this, void 0, void 0, function* () {
                        if (!util.isNullOrUndefined(locatorName)) {
                            yield GetCurrentElementInfo_1.GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                            if ((GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                                if (retryCount != 1)
                                    GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                yield GetHealedElementInfo_1.GetHealedElementInfo.getElementInfo(GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            retryCount += 1;
                            if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                                if (((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable_1.GlobalVariable.error).toString()).includes('still not visible')) {
                                    console.log('-----AUTOHEAL INITIATED-----');
                                    GlobalVariable_1.GlobalVariable.ElementMap.set('autoUpdate', 0);
                                    yield AutoHeal_1.AutoHeal.startHealing(locatorName).then(() => __awaiter(this, void 0, void 0, function* () {
                                        if (GlobalVariable_1.GlobalVariable.HealedElement != 'NA') {
                                            yield this.grabCssPropertyFromAll(GlobalVariable_1.GlobalVariable.HealedElement, cssProperty, locatorName, timeout, retryCount);
                                            resolve(GlobalVariable_1.GlobalVariable.postHTMLFromAll);
                                        }
                                        else
                                            reject('Healed xpath does not meet the minimum confidence score bar');
                                    })).catch(error => {
                                        console.log('----Error in healing process----');
                                        reject(error);
                                    });
                                }
                                else {
                                    console.log('----Step failed due to : ' + GlobalVariable_1.GlobalVariable.error);
                                    reject(GlobalVariable_1.GlobalVariable.error);
                                }
                            }
                            else if (!result && retryCount > 1) {
                                reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable_1.GlobalVariable.HealedElement);
                            }
                            else if (!result && retryCount == 1 && GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                console.log('*************Healing NA**************');
                                reject('Test Step Failed and no healing was performed as element info was not present ');
                            }
                            else {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.waitForVisible(locator);
                                GlobalVariable_1.GlobalVariable.postHTMLFromAll = yield I.grabCssPropertyFromAll(locator, cssProperty);
                                if (retryCount == 2 || GlobalVariable_1.GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        yield GetHealedElementInfo_1.GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable_1.GlobalVariable.HealedElement = locator;
                                    yield GetCurrentElementInfo_1.GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                yield I.wait(10);
                                resolve(GlobalVariable_1.GlobalVariable.postHTMLFromAll);
                            }
                        }
                        else {
                            if (result) {
                                yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                                yield I.waitForVisible(locator);
                                GlobalVariable_1.GlobalVariable.postHTMLFromAll = yield I.grabCssPropertyFromAll(locator, cssProperty);
                                resolve(GlobalVariable_1.GlobalVariable.postHTMLFromAll);
                            }
                            else {
                                reject('-----Test Step Failed and no AUTOHEAL adopted-----');
                            }
                        }
                    })).catch((error) => {
                        reject(error);
                    });
                }
            }));
        });
    }
}
exports.autoHeal = autoHeal;
