"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.nonHeal = void 0;
const util = require('util');
const http = __importStar(require("http"));
const fs = __importStar(require("fs"));
const codeceptjs_1 = require("codeceptjs");
const { I } = inject();
class nonHeal extends codeceptjs_1.Helper {
    /**
     * Scrolls element into viewport, waits for that element to become visible & clickable, then performs click action.
     * Throws error if any of the action fails in two attempts.
     * The second parameter is a context (CSS or XPath locator) to narrow the search.
     * ```js
     * // simple link
     * z.click('Logout');
     * // button of form
     * z.click('Submit');
     * // CSS button
     * z.click('#form input[type=submit]');
     * // XPath
     * z.click('//form/*[@type=submit]');
     * // link in context
     * z.click('Logout', '#nav');
     * // using strict locator
     * z.click({css: 'nav a.login'});
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {string} context (optional, null by default) element to search in CSS|XPath|Strict locator.
     */
    static click(locator, context) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                    yield I.waitForElement(locator);
                    yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForVisible(locator);
                    yield I.waitForClickable(locator);
                    if (util.isNullOrUndefined(context)) {
                        yield I.click(locator);
                    }
                    else {
                        yield I.click(locator, context);
                    }
                }
                else {
                    if (util.isNullOrUndefined(context)) {
                        yield I.click(locator);
                    }
                    else {
                        yield I.click(locator, context);
                    }
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                        yield I.waitForElement(locator);
                        yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                        yield I.waitForVisible(locator);
                        yield I.waitForClickable(locator);
                        if (util.isNullOrUndefined(context)) {
                            yield I.click(locator);
                        }
                        else {
                            yield I.click(locator, context);
                        }
                    }
                    else {
                        if (util.isNullOrUndefined(context)) {
                            yield I.click(locator);
                        }
                        else {
                            yield I.click(locator, context);
                        }
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("click : Error while clicking on element located by : " + locator);
                }
            }
        });
    }
    /**
     * Scrolls element into viewport, waits for that element to become visible & enabled, then clears field value.
     * Throws error if any of the action fails in two attempts.
     * ```js
     * z.clearField('Email');
     * z.clearField('user[email]');
     * z.clearField('#email');
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     */
    static clearField(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                    yield I.waitForElement(locator);
                    yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForVisible(locator);
                    yield I.waitForEnabled(locator);
                    yield I.clearField(locator);
                }
                else {
                    yield I.clearField(locator);
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                        yield I.waitForElement(locator);
                        yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                        yield I.waitForVisible(locator);
                        yield I.waitForEnabled(locator);
                        yield I.clearField(locator);
                    }
                    else {
                        yield I.clearField(locator);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("clearField : Error while clearing value in element located by : " + locator);
                }
            }
        });
    }
    /**
     * Waits for element to be visible, Clears element value & fills given value.
     * Throws error if any of the action fails in two attempts.
     * ```js
     * // by label
     * z.fillField('Email', 'hello@world.com');
     * // by name
     * z.fillField('password', secret('123456'));
     * // by CSS
     * z.fillField('form#login input[name=username]', 'John');
     * // or by strict locator
     * z.fillField({css: 'form#login input[name=username]'} 'John');
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {string} value
     */
    static fillField(locator, value) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.clearField(locator);
                yield I.fillField(locator, value);
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    yield this.clearField(locator);
                    yield I.fillField(locator, value);
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("fillField : Error while filling value in element located by : " + locator);
                }
            }
        });
    }
    /**
     * Checks for the visibility of text, if optional context parameter given then searches for the visibility of text on the locator element.
     * Throws error if any of the action fails in two attempts.
     * ```js
     * z.see('Welcome'); // text welcome on a page
     * z.see('Welcome', '.content'); // text inside .content div
     * z.see('Register', {css: 'form.register'}); // use strict locator
     * ```
     * @param {string} text
     * @param {string} context optional, element located by CSS|Xpath|strict locator in which to search for text.
     */
    static see(text, context) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (util.isNullOrUndefined(context)) {
                    yield I.wait(2);
                    yield I.see(text);
                }
                else {
                    yield I.waitForVisible(context);
                    yield I.see(text, context);
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (util.isNullOrUndefined(context)) {
                        yield I.wait(2);
                        yield I.see(text);
                    }
                    else {
                        yield I.waitForVisible(context);
                        yield I.see(text, context);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("see : Error while checking visibility of : " + text);
                }
            }
        });
    }
    /**
     * Checks the visibilty of an element.
     * Throws error if any of the action fails in two attempts.
     * ```js
     * z.seeElement('#modal');
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     */
    static seeElement(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield I.waitForVisible(locator);
                yield I.seeElement(locator);
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    yield I.waitForVisible(locator);
                    yield I.seeElement(locator);
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("seeElement : Error while checking visibility of : " + locator);
                }
            }
        });
    }
    /**
     * Opens a webpage in browser with given url.
     * ```js
     * z.amOnPage('/'); // opens main page of website
     * z.amOnPage('https://github.com'); // opens github
     * z.amOnPage('/login'); // opens a login page
     * ```
     * @param {string} url
     */
    static amOnPage(url) {
        return __awaiter(this, void 0, void 0, function* () {
            yield I.amOnPage(url);
        });
    }
    /**
     * Asserts that two values are equal. If they are not,an assertion error is thrown.
     * ```js
     * z.assertEqual(expected,actual)
     * ```
     * @param {any} expected
     * @param {any} actual
     */
    static assertEqual(expected, actual, msg) {
        return __awaiter(this, void 0, void 0, function* () {
            if (util.isNullOrUndefined(msg))
                yield I.assertEqual(expected, actual);
            else
                yield I.assertEqual(expected, actual, msg);
        });
    }
    /**
     * Returns all window handles. Useful for referencing a specific handle when calling I.switchToWindow(handle)
     * ```js
     * const windows = await I.grabAllWindowHandles();
     * ```
     * @returns {Array} windows
     */
    static grabAllWindowHandles() {
        return __awaiter(this, void 0, void 0, function* () {
            let windows = yield I.grabAllWindowHandles();
            return windows;
        });
    }
    /**
     * Returns current window handle
     * ```js
     * const window = await z.grabCurrentWindowHandle();
     * ```
     * @returns {any} window
     */
    static grabCurrentWindowHandle() {
        return __awaiter(this, void 0, void 0, function* () {
            let window = yield I.grabCurrentWindowHandle();
            return window;
        });
    }
    /**
     * Grabs the number of visible elements
     * ```js
     * let numOfElements = await z.grabNumberOfVisibleElements('p');
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @returns {any} totalElements
     */
    static grabNumberOfVisibleElements(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            let totalElements = yield I.grabNumberOfVisibleElements(locator);
            return totalElements;
        });
    }
    /**
     * Open new tab and switch to it.
     * ```js
     * z.openNewTab();
     * ```
     */
    static openNewTab() {
        return __awaiter(this, void 0, void 0, function* () {
            yield I.openNewTab();
        });
    }
    /**
     * Presses a key or a key in combination with modifier keys in the browser (on a focused element).
     * ```js
    * z.pressKey('Backspace');
    * ```
    *
    * To press a key in combination with modifier keys, pass the sequence as an array. All modifier keys (`'Alt'`, `'Control'`, `'Meta'`, `'Shift'`) will be released afterwards.
    *
    * ```js
    * z.pressKey(['Control', 'Z']);
    * ```
    *
    * For specifying operation modifier key based on operating system it is suggested to use `'CommandOrControl'`.
    * This will press `'Command'` (also known as `'Meta'`) on macOS machines and `'Control'` on non-macOS machines.
    *
    * ```js
    * z.pressKey(['CommandOrControl', 'Z']);
    * ```
    *
    * Some of the supported key names are:
    * - `'AltLeft'` or `'Alt'`
    * - `'AltRight'`
    * - `'ArrowDown'`
    * - `'ArrowLeft'`
    * - `'ArrowRight'`
    * - `'ArrowUp'`
    * - `'Backspace'`
    * - `'Clear'`
    * - `'ControlLeft'` or `'Control'`
    * - `'ControlRight'`
    * - `'Command'`
    * - `'CommandOrControl'`
    * - `'Delete'`
    * - `'End'`
    * - `'Enter'`
    * - `'Escape'`
    * - `'F1'` to `'F12'`
    * - `'Home'`
    * - `'Insert'`
    * - `'MetaLeft'` or `'Meta'`
    * - `'MetaRight'`
    * - `'Numpad0'` to `'Numpad9'`
    * - `'NumpadAdd'`
    * - `'NumpadDecimal'`
    * - `'NumpadDivide'`
    * - `'NumpadMultiply'`
    * - `'NumpadSubtract'`
    * - `'PageDown'`
    * - `'PageUp'`
    * - `'Pause'`
    * - `'Return'`
    * - `'ShiftLeft'` or `'Shift'`
    * - `'ShiftRight'`
    * - `'Space'`
    * - `'Tab'`
     * @param {string|Array} key
     */
    static pressKey(key) {
        return __awaiter(this, void 0, void 0, function* () {
            yield I.pressKey(key);
        });
    }
    /**
     * Reload the current page.
     * ```js
     * z.refreshPage();
     * ```
     */
    static refreshPage() {
        return __awaiter(this, void 0, void 0, function* () {
            yield I.refreshPage();
        });
    }
    /**
     * Saves a screenshot to ouput folder (set in codecept.json or codecept.conf.js).
     * Filename is relative to output folder. Optionally resize the window to the full available page scrollHeight and scrollWidth
     * to capture the entire page by passing true in as the second argument.
     * ```js
     * z.saveScreenshot('debug.png');
     * z.saveScreenshot('debug.png', true) //resizes to available scrollHeight and scrollWidth before taking screenshot
     * ```
     * @param {string} fileName
     * @param {boolean} fullPage optional
     */
    static saveScreenshot(fileName, fullPage) {
        return __awaiter(this, void 0, void 0, function* () {
            if (util.isNullOrUndefined(fullPage)) {
                yield I.saveScreenshot(fileName);
            }
            else {
                yield I.saveScreenshot(fileName, fullPage);
            }
        });
    }
    /**
     * Switches frame or in case of null locator reverts to parent.
     * ```js
     * z.switchTo('iframe'); // switch to first iframe
     * z.switchTo(); // switch back to main page
     * ```
     * @param {string} locator optional, CSS|XPath|strict locator.
     */
    static switchTo(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!util.isNullOrUndefined(locator)) {
                yield I.switchTo(locator);
            }
            else {
                yield I.switchTo();
            }
        });
    }
    /**
     * Switch focus to a particular tab by its number. It waits tabs loading and then switch tab.
     * ```js
     * z.switchToNextTab();
     * z.switchToNextTab(2);
     * ```
     * @param {number} tabNum optional, number of tabs to switch forward, default: 1.
     * @param {number} timeout optional, time in seconds to wait.
     */
    static switchToNextTab(tabNum, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            yield I.switchToNextTab(tabNum, timeout);
        });
    }
    /**
     * Switch to the window with a specified handle.
     * ```js
     * const windows = await z.grabAllWindowHandles();
     * //  do something
     * await z.switchToWindow( windows[0] );
     *
     * const window = await z.grabCurrentWindowHandle();
     * //  do something
     * await z.switchToWindow( window );
     * ```
     * @param {string} window
     */
    static switchToWindow(window) {
        return __awaiter(this, void 0, void 0, function* () {
            yield I.switchToWindow(window);
        });
    }
    /**
     * Pauses execution for a number of seconds.
     * ```js
     * z.wait(2); // wait 2 secs
     * ```
     * @param {number} timeout
     */
    static wait(timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            yield I.wait(timeout);
        });
    }
    /**
     * Waits for the visibility of element & for the specified value to be in value attribute.
     * ```js
     * z.waitForValue('//input', "GoodValue");
     * ```
     * @param {string} locator input field locator.
     * @param {string} value
     * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
     */
    static waitForValue(locator, value, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield I.waitForVisible(locator);
                if (util.isNullOrUndefined(timeout)) {
                    yield I.waitForValue(locator, value);
                }
                else {
                    yield I.waitForValue(locator, value, timeout);
                }
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield this.generateLog(timestamp, error);
                yield I.saveScreenshot(timestamp.toString() + ".png");
                throw new Error("waitForValue : Error while waiting for value in element located by : " + locator);
            }
        });
    }
    /**
     * Waits for the visibility of element.
     * ```js
     * z.waitForVisible('#popup');
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
     */
    static waitForVisible(locator, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!util.isNullOrUndefined(timeout)) {
                    yield I.waitForVisible(locator, timeout);
                }
                else {
                    yield I.waitForVisible(locator);
                }
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield this.generateLog(timestamp, error);
                yield I.saveScreenshot(timestamp.toString() + ".png");
                throw new Error("waitForVisible : Error while waiting for visibility of nn element located by : " + locator);
            }
        });
    }
    /**
     * Waits for a text to appear.
     * Element can be located by CSS or XPath. Narrow down search results by providing context.
     * ```js
     * z.waitForText('Thank you, form has been submitted');
     * z.waitForText('Thank you, form has been submitted', 5, '#modal');
     * ```
     * @param {string} text
     * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
     * @param {string} context optional, element located by CSS|XPath|strict locator.
     
     */
    static waitForText(text, timeout, context) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!util.isNullOrUndefined(context) && util.isNullOrUndefined(timeout)) {
                    yield I.scrollIntoView(context, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForText(text, undefined, context);
                }
                else if (!util.isNullOrUndefined(timeout) && util.isNullOrUndefined(context)) {
                    yield I.waitForText(text, timeout);
                }
                else if (!util.isNullOrUndefined(timeout) && !util.isNullOrUndefined(context)) {
                    yield I.scrollIntoView(context, { block: "start" });
                    yield I.waitForText(text, timeout, context);
                }
                else if (util.isNullOrUndefined(context) && util.isNullOrUndefined(timeout)) {
                    yield I.waitForText(text);
                }
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield this.generateLog(timestamp, error);
                yield I.saveScreenshot(timestamp.toString() + ".png");
                throw new Error("waitForText : Error while waiting for text : " + text);
            }
        });
    }
    /**
     * Waits for an element to be removed or become invisible on a page.
     * Element can be located by CSS or XPath.
     * ```js
     * z.waitForInvisible('#popup');
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
     */
    static waitForInvisible(locator, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (util.isNullOrUndefined(timeout)) {
                    yield I.waitForInvisible(locator);
                }
                else {
                    yield I.waitForInvisible(locator, timeout);
                }
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield this.generateLog(timestamp, error);
                yield I.saveScreenshot(timestamp.toString() + ".png");
                throw new Error("waitForInvisible : Error while waiting invisibility of element located by: " + locator);
            }
        });
    }
    /**
     * Waits for an element to become enabled.
     * Element can be located by CSS or XPath.
     * ```js
     * z.waitForEnabled('#popup');
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
     */
    static waitForEnabled(locator, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (util.isNullOrUndefined(timeout)) {
                    yield I.waitForEnabled(locator);
                }
                else {
                    yield I.waitForEnabled(locator, timeout);
                }
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield this.generateLog(timestamp, error);
                yield I.saveScreenshot(timestamp.toString() + ".png");
                throw new Error("waitForEnabled : Error while waiting for element to be enabled located by: " + locator);
            }
        });
    }
    /**
     * Waits for an element to be present on page.
     * Element can be located by CSS or XPath.
     * ```js
     * z.waitForElement('.btn.continue');
     * z.waitForElement('.btn.continue', 5); // wait for 5 secs
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
     */
    static waitForElement(locator, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!util.isNullOrUndefined(timeout)) {
                    yield I.waitForElement(locator, timeout);
                }
                else {
                    yield I.waitForElement(locator);
                }
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield this.generateLog(timestamp, error);
                yield I.saveScreenshot(timestamp.toString() + ".png");
                throw new Error("waitForElement : Error while waiting for element to be present located by: " + locator);
            }
        });
    }
    /**
     * Waits for an element to be clickable.
     * Element can be located by CSS or XPath.
     * ```js
     * z.waitForClickable('.btn.continue');
     * z.waitForClickable('.btn.continue', 5); // wait for 5 secs
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
     */
    static waitForClickable(locator, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (util.isNullOrUndefined(timeout)) {
                    yield I.waitForClickable(locator);
                }
                else {
                    yield I.waitForClickable(locator, timeout);
                }
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield this.generateLog(timestamp, error);
                yield I.saveScreenshot(timestamp.toString() + ".png");
                throw new Error("waitForClickable : Error while waiting for element to be clickable located by: " + locator);
            }
        });
    }
    /**
     * Waits for an element to be visible.
     * Element can be located by CSS or XPath.
     * ```js
     * z.attachFile('Avatar', 'data/avatar.jpg');
     * z.attachFile('form input[name=avatar]', 'data/avatar.jpg');
     * ```
     * @param {string} locator label|name|CSS|XPath|strict locator.
     * @param {string} filepath local file path relative to codecept.json config file
     */
    static attachFile(locator, filepath) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                    yield I.waitForElement(locator);
                    yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                    yield I.attachFile(locator, filepath);
                }
                else {
                    yield I.attachFile(locator, filepath);
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                        yield I.waitForElement(locator);
                        yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                        yield I.attachFile(locator, filepath);
                    }
                    else {
                        yield I.attachFile(locator, filepath);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("attachFile : Error while uploading file to the element located by: " + locator);
                }
            }
        });
    }
    /**
     * Switch focus to a particular tab by its number. It waits tabs loading and then switch tab.
     * ```js
     * z.switchToPreviousTab();
     * z.switchToPreviousTab(2);
     * ```
     * @param {number} tabNum optional, number of tabs to switch backward, default: 1.
     * @param {number} timeout optional, time in seconds to wait.
     */
    static switchToPreviousTab(tabNum, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            yield I.switchToPreviousTab(tabNum, timeout);
        });
    }
    /**
     * Close current tab.
     * ```js
     * z.closeCurrentTab();
     * ```
     */
    static closeCurrentTab() {
        return __awaiter(this, void 0, void 0, function* () {
            yield I.closeCurrentTab();
        });
    }
    /**
     * Close all tabs except for the current one.
     * ```js
     * z.closeOtherTabs();
     * ```
     */
    static closeOtherTabs() {
        return __awaiter(this, void 0, void 0, function* () {
            yield I.closeOtherTabs();
        });
    }
    /**
     * Scroll element into viewport.
     * ```js
     * z.scrollIntoView('#submit');
     * z.scrollIntoView('#submit', { behavior: "smooth", block: "center", inline: "center" });
     * z.scrollIntoView('#submit', true);
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {any} alignTop optional, or scrollIntoViewOptions (optional),
     * see https://developer.mozilla.org/en-US/docs/Web/API/Element/scrollIntoView.
     
     */
    static scrollIntoView(locator, alignTop) {
        return __awaiter(this, void 0, void 0, function* () {
            if (alignTop == null || alignTop == undefined) {
                yield I.scrollIntoView(locator, alignTop);
            }
            else {
                yield I.scrollIntoView(locator, { block: "start" });
            }
        });
    }
    /**
     * Retrieves a text from an element located by CSS or XPath and returns it to test.
     * Resumes test execution, so should be used inside static async with await operator.
     * If multiple elements found returns an array of texts.
     * ```js
     * let pin = await z.grabTextFrom('#pin');
     * ```
     * @param {string} locator element located by CSS|XPath|strict locator.
     * @returns {string|string[]} text
     */
    static grabTextFrom(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            let text = "";
            try {
                if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                    yield I.waitForElement(locator);
                    yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                    text = yield I.grabTextFrom(locator);
                }
                else {
                    text = yield I.grabTextFrom(locator);
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                        yield I.waitForElement(locator);
                        yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                        text = yield I.grabTextFrom(locator);
                    }
                    else {
                        text = yield I.grabTextFrom(locator);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("grabTextFrom : Error while fetching text from element located by: " + locator);
                }
            }
            return text;
        });
    }
    static grabTextFromAll(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            let text = [];
            try {
                if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                    yield I.waitForElement(locator);
                    yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                    text = yield I.grabTextFromAll(locator);
                }
                else {
                    text = yield I.grabTextFromAll(locator);
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                        yield I.waitForElement(locator);
                        yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                        text = yield I.grabTextFromAll(locator);
                    }
                    else {
                        text = yield I.grabTextFromAll(locator);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("grabTextFromAll : Error while fetching text from element located by: " + locator);
                }
            }
            return text;
        });
    }
    /**
     * Retrieves an attribute from an element located by CSS or XPath and returns it to test. An array as a result will be returned if there are more than one matched element.
     * Resumes test execution, so should be used inside static async with await operator.
     * ```js
     * let hint = await z.grabAttributeFrom('#tooltip', 'title');
     * ```
     * @param {string} locator element located by CSS|XPath|strict locator.
     * @param {string} attribute attribute name.
     * @returns {string} text
     */
    static grabAttributeFrom(locator, attribute, elementLocatorStrategy = 'xpath') {
        return __awaiter(this, void 0, void 0, function* () {
            let value = "";
            try {
                if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                    yield I.waitForElement(locator);
                    yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForVisible(locator);
                    value = yield this.grabPropertyOrAttributeValue(locator, attribute, elementLocatorStrategy);
                }
                else {
                    value = yield this.grabPropertyOrAttributeValue(locator, attribute, elementLocatorStrategy);
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                        yield I.waitForElement(locator);
                        yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                        yield I.waitForVisible(locator);
                        value = yield this.grabPropertyOrAttributeValue(locator, attribute, elementLocatorStrategy);
                    }
                    else {
                        value = yield this.grabPropertyOrAttributeValue(locator, attribute, elementLocatorStrategy);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("grabAttributeFrom : Error while fetching attribute '" + attribute + "' from element located by: " + locator);
                }
            }
            return value;
        });
    }
    static grabAttributeFromAll(locator, attribute, elementLocatorStrategy = 'xpath') {
        return __awaiter(this, void 0, void 0, function* () {
            let value;
            try {
                if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                    yield I.waitForElement(locator);
                    yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForVisible(locator);
                    value = yield this.grabPropertyOrAttributeValueFromAll(locator, attribute, elementLocatorStrategy);
                }
                else {
                    value = yield this.grabPropertyOrAttributeValueFromAll(locator, attribute, elementLocatorStrategy);
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                        yield I.waitForElement(locator);
                        yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                        yield I.waitForVisible(locator);
                        value = yield this.grabPropertyOrAttributeValueFromAll(locator, attribute, elementLocatorStrategy);
                    }
                    else {
                        value = yield this.grabPropertyOrAttributeValueFromAll(locator, attribute, elementLocatorStrategy);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("grabAttributeFromAll : Error while fetching attribute '" + attribute + "' from element located by: " + locator);
                }
            }
            return value;
        });
    }
    /**
         * Scrolls element into viewport, waits for that element to become visible & clickable. Also checks if element is already checked.
         * Then selects a checkbox or radio button.
         *
         * The second parameter is a context (CSS or XPath locator) to narrow the search.
         * ```js
         * z.checkOption('#agree');
         * z.checkOption('I Agree to Terms and Conditions');
         * z.checkOption('agree', '//form');
         * ```
         * @param {string} locator checkbox located by label | name | CSS | XPath | strict locator.
         * @param {string} context optional element located by CSS | XPath | strict locator.
         */
    static checkOption(locator, context) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                    yield I.waitForElement(locator);
                    yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForVisible(locator);
                    yield I.waitForClickable(locator);
                    yield I.dontSeeCheckboxIsChecked(locator);
                }
                if (!util.isNullOrUndefined(context)) {
                    yield I.checkOption(locator, context);
                }
                else {
                    yield I.checkOption(locator);
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                        yield I.waitForElement(locator);
                        yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                        yield I.waitForVisible(locator);
                        yield I.waitForClickable(locator);
                        yield I.dontSeeCheckboxIsChecked(locator);
                    }
                    if (!util.isNullOrUndefined(context)) {
                        yield I.checkOption(locator, context);
                    }
                    else {
                        yield I.checkOption(locator);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("checkOption : Error while selecting element located by: " + locator);
                }
            }
        });
    }
    /**
     * Checks that a text is not present on a page, if optional context parameter given then searches for the invisibility of text on the locator element.
     * Use context parameter to narrow down the search.
     * Throws error if any of the action fails in two attempts.
     * ```js
     * z.dontSee('Login'); // assume we are already logged in.
     * z.dontSee('Login', '.nav'); // no login inside .nav element
     * ```
     * @param {string} text which is not present.
     * @param {string} context optional, element located by CSS|Xpath|strict locator in which to search for text.
     */
    static dontSee(text, context) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!util.isNullOrUndefined(context)) {
                    yield I.dontSee(text, context);
                }
                else {
                    yield I.dontSee(text);
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (!util.isNullOrUndefined(context)) {
                        yield I.dontSee(text, context);
                    }
                    else {
                        yield I.dontSee(text);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("dontSee : Error while checking invisibility of element by: " + text);
                }
            }
        });
    }
    /**
     * Checks that element is not visible (or in DOM)
     * Throws error if any of the action fails in two attempts.
     * ```js
     * z.seeElement('#modal');
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     */
    static dontSeeElement(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield I.dontSeeElement(locator);
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    yield I.dontSeeElement(locator);
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("dontSeeElement : Error while checking invisibility of element by: " + locator);
                }
            }
        });
    }
    /**
     * Scrolls element into viewport, waits for that element to become visible & clickable, then performs double click action.
     * Throws error if any of the action fails in two attempts.
     * The second parameter is a context (CSS or XPath locator) to narrow the search.
     * ```js
     * z.doubleClick('Edit');
     * z.doubleClick('Edit', '.actions');
     * z.doubleClick({css: 'button.accept'});
     * z.doubleClick('.btn.edit');
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {string} context (optional, null by default) element to search in CSS|XPath|Strict locator.
     */
    static doubleClick(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                    yield I.waitForElement(locator);
                    yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForVisible(locator);
                    yield I.waitForClickable(locator);
                }
                yield I.doubleClick(locator);
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                        yield I.waitForElement(locator);
                        yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                        yield I.waitForVisible(locator);
                        yield I.waitForClickable(locator);
                    }
                    yield I.doubleClick(locator);
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("doubleClick : Error while double clicking on element located by : " + locator);
                }
            }
        });
    }
    /**
     * Scrolls element into viewport, waits for that element to become visible & clickable, then Drag an item to a destination element.
     * Throws error if any of the action fails in two attempts.
     *
     * ```js
     * z.dragAndDrop('#dragHandle', '#container');
     * ```
     * @param {string} srcLocator CSS|XPath|strict locator.
     * @param {string} destLocator CSS|XPath|Strict locator.
     */
    static dragAndDrop(srcLocator, destLocator) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (srcLocator.startsWith("/") || srcLocator.startsWith(".") || srcLocator.startsWith("#") || srcLocator.startsWith("{")) {
                    yield I.waitForElement(srcLocator);
                    yield I.scrollIntoView(srcLocator, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForVisible(srcLocator);
                    yield I.waitForClickable(srcLocator);
                    yield I.waitForVisible(destLocator);
                    yield I.dragAndDrop(srcLocator, destLocator);
                }
                else {
                    yield I.dragAndDrop(srcLocator, destLocator);
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (srcLocator.startsWith("/") || srcLocator.startsWith(".") || srcLocator.startsWith("#") || srcLocator.startsWith("{")) {
                        yield I.waitForElement(srcLocator);
                        yield I.scrollIntoView(srcLocator, { behavior: "auto", block: "center", inline: "center" });
                        yield I.waitForVisible(srcLocator);
                        yield I.waitForClickable(srcLocator);
                        yield I.waitForVisible(destLocator);
                        yield I.dragAndDrop(srcLocator, destLocator);
                    }
                    else {
                        yield I.dragAndDrop(srcLocator, destLocator);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("dragAndDrop : Error while performing drag and drop from element located by : " + srcLocator + " to " + destLocator);
                }
            }
        });
    }
    /**
     * Executes sync script on a page.
     * Pass arguments to function as additional parameters.
     * Will return execution result to a test.
     * In this case you should use static async function and await to receive results.
     *
     * Example with jQuery DatePicker:
     *
     * ```js
     * // change date of jQuery DatePicker
     * z.executeScript(function() {
     *   // now we are inside browser context
     *   $('date').datetimepicker('setDate', new Date());
     * });
     * ```
     * Can return values. Don't forget to use `await` to get them.
     *
     * ```js
     * let date = await z.executeScript(function(el)
     *   // only basic types can be returned
     *   return $(el).datetimepicker('getDate').tostring();
     * } '#date'); // passing jquery selector
     * ```
     * @param {string|function} fn function to be executed in browser context.
     * @param {...any} args to be passed to function.
     * */
    static executeScript(fn, ...args) {
        return __awaiter(this, void 0, void 0, function* () {
            let retunrtype = yield I.executeScript(fn, args);
            return retunrtype;
        });
    }
    /**
     * Scrolls element into viewport, waits for that element to become visible & clickable, then selects an option in a drop-down select.
     * Throws error if any of the action fails in two attempts.
     *
     * ```js
     * z.selectOption('Choose Plan', 'Monthly'); // select by label
     * z.selectOption('subscription', 'Monthly'); // match option by text
     * z.selectOption('subscription', '0'); // or by value
     * z.selectOption('//form/select[@name=account]','Premium');
     * z.selectOption('form select[name=account]', 'Premium');
     * z.selectOption({css: 'form select[name=account]'} 'Premium');
     * ```
     * Provide an array for the second argument to select multiple options.
     * ```js
     * z.selectOption('Which OS do you use?', ['Android', 'iOS']);
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {string|string[]} option CSS|XPath|Strict locator.
     */
    static selectOption(locator, option) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                    yield I.waitForElement(locator);
                    yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForVisible(locator);
                    yield I.waitForClickable(locator);
                    yield I.selectOption(locator, option);
                }
                else {
                    yield I.selectOption(locator, option);
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                        yield I.waitForElement(locator);
                        yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                        yield I.waitForVisible(locator);
                        yield I.waitForClickable(locator);
                        yield I.selectOption(locator, option);
                    }
                    else {
                        yield I.selectOption(locator, option);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("selectOption : Error while selecting value on element located by :  " + locator);
                }
            }
        });
    }
    /**
     * Checks for element to be present, scrolls into view & Moves cursor to element matched by locator.
     * Extra shift can be set with offsetX and offsetY options.
     *
     * ```js
     * z.moveCursorTo('.tooltip');
     * z.moveCursorTo('#submit', 5,5);
     * ```
     *
     * @param {CodeceptJS.LocatorOrString} locator located by CSS|XPath|strict locator.
     * @param {number} [offsetX=0] (optional, `0` by default) X-axis offset.
     * @param {number} [offsetY=0] (optional, `0` by default) Y-axis offset.
     *
     *
     */
    static moveCursorTo(locator, offsetX, offsetY) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                    yield I.waitForElement(locator);
                    yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                }
                if (util.isNullOrUndefined(offsetX) && util.isNullOrUndefined(offsetY)) {
                    yield I.moveCursorTo(locator);
                }
                else if (util.isNullOrUndefined(offsetX) && !(util.isNullOrUndefined(offsetY))) {
                    yield I.moveCursorTo(locator, 0, offsetY);
                }
                else if (!(util.isNullOrUndefined(offsetX)) && util.isNullOrUndefined(offsetY)) {
                    yield I.moveCursorTo(locator, offsetX, 0);
                }
                else {
                    yield I.moveCursorTo(locator, offsetX, offsetY);
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                        yield I.waitForElement(locator);
                        yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                    }
                    if (util.isNullOrUndefined(offsetX) && util.isNullOrUndefined(offsetY)) {
                        yield I.moveCursorTo(locator);
                    }
                    else if (util.isNullOrUndefined(offsetX) && !(util.isNullOrUndefined(offsetY))) {
                        yield I.moveCursorTo(locator, 0, offsetY);
                    }
                    else if (!(util.isNullOrUndefined(offsetX)) && util.isNullOrUndefined(offsetY)) {
                        yield I.moveCursorTo(locator, offsetX, 0);
                    }
                    else {
                        yield I.moveCursorTo(locator, offsetX, offsetY);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("moveCursorTo : Error while moving cursor to element located by :  " + locator);
                }
            }
        });
    }
    /**
     * Checks for element to be present, scrolls into view, waits for element to be visible and retrieves a value from a form element located by CSS or XPath, returns it to test.
     * Resumes test execution, so **should be used inside async function with `await`** operator.
     *
     * ```js
     * let email = await z.grabValueFrom('input[name=email]');
     * ```
     * @param {CodeceptJS.LocatorOrString} locator field located by label|name|CSS|XPath|strict locator.
     * @returns {Promise<any>} attribute value
     *
     */
    static grabValueFrom(locator, elementLocatorStrategy = 'xpath') {
        return __awaiter(this, void 0, void 0, function* () {
            let text = "";
            try {
                if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                    yield I.waitForElement(locator);
                    yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForVisible(locator);
                    text = yield this.grabPropertyOrAttributeValue(locator, 'value', elementLocatorStrategy);
                }
                else {
                    text = yield this.grabPropertyOrAttributeValue(locator, 'value', elementLocatorStrategy);
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                        yield I.waitForElement(locator);
                        yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                        yield I.waitForVisible(locator);
                        text = yield this.grabPropertyOrAttributeValue(locator, 'value', elementLocatorStrategy);
                    }
                    else {
                        text = yield this.grabPropertyOrAttributeValue(locator, 'value', elementLocatorStrategy);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("grabValueFrom : Error while grabbing value from element located by :  " + locator);
                }
            }
            return text;
        });
    }
    static grabValueFromAll(locator, elementLocatorStrategy = 'xpath') {
        return __awaiter(this, void 0, void 0, function* () {
            let text;
            try {
                if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                    yield I.waitForElement(locator);
                    yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForVisible(locator);
                    text = yield this.grabPropertyOrAttributeValueFromAll(locator, 'value', elementLocatorStrategy);
                }
                else {
                    text = yield this.grabPropertyOrAttributeValueFromAll(locator, 'value', elementLocatorStrategy);
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                        yield I.waitForElement(locator);
                        yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                        yield I.waitForVisible(locator);
                        text = yield this.grabPropertyOrAttributeValueFromAll(locator, 'value', elementLocatorStrategy);
                    }
                    else {
                        text = yield this.grabPropertyOrAttributeValueFromAll(locator, 'value', elementLocatorStrategy);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("grabValueFromAll : Error while grabbing value from element located by :  " + locator);
                }
            }
            return text;
        });
    }
    /**
     * Dismisses the active JavaScript popup, as created by window.alert|window.confirm|window.prompt.
     * ```js
     * await z.cancelPopup();
     * ```
     */
    static cancelPopup() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield I.cancelPopup();
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield this.generateLog(timestamp, error);
                yield I.saveScreenshot(timestamp.toString() + ".png");
                throw new Error("cancelPopup : Error while cancelling popup.");
            }
        });
    }
    /**
     * Accepts the active JavaScript native popup window, as created by window.alert|window.confirm|window.prompt.
     * Don't confuse popups with modal windows, as created by [various
     * libraries](http://jster.net/category/windows-modals-popups).
     * ```js
     * await z.acceptPopup();
     * ```
     */
    static acceptPopup() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield I.acceptPopup();
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield this.generateLog(timestamp, error);
                yield I.saveScreenshot(timestamp.toString() + ".png");
                throw new Error("acceptPopup : Error while accepting popup.");
            }
        });
    }
    /**
     * Checks that the active JavaScript popup, as created by `window.alert|window.confirm|window.prompt`, contains the
     * given string.
     *
     * @param {string} text value to check.
     *
     * ```js
     * await z.seeInPopup("value");
     * ```
     *
     */
    static seeInPopup(text) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield I.seeInPopup(text);
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield this.generateLog(timestamp, error);
                yield I.saveScreenshot(timestamp.toString() + ".png");
                throw new Error("seeInPopup : Error while checking text in popup.");
            }
        });
    }
    /**
     * Gets a cookie object by name.
     * If none provided gets all cookies.
     * Resumes test execution, so **should be used inside async function with `await`** operator.
     *
     * ```js
     * let cookie = await z.grabCookie('auth');
     * assert(cookie.value, '123456');
     * ```
     *
     * @param {?string} [name=null] cookie name.
     * @returns {Promise<string>} attribute value
     *
     */
    static grabCookie(name) {
        return __awaiter(this, void 0, void 0, function* () {
            let value = "";
            try {
                value = yield I.grabCookie(name);
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield this.generateLog(timestamp, error);
                yield I.saveScreenshot(timestamp.toString() + ".png");
                throw new Error("grabCookie : Error fetching cookie.");
            }
            return value;
        });
    }
    /**
     * Get current URL from browser
     * Resumes test execution, so **should be used inside async function with `await`** operator.
     *
     * ```js
     * let url = await z.grabCurrentUrl();
     * ```
     *
     */
    static grabCurrentUrl() {
        return __awaiter(this, void 0, void 0, function* () {
            let url = "";
            try {
                url = yield I.grabCurrentUrl();
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield I.saveScreenshot(timestamp.toString() + ".png");
                yield this.generateLog(timestamp, error);
                throw new Error("grabCurrentUrl : Error fetching current url.");
            }
            return url;
        });
    }
    /**
    * Grab number of open tabs.
    * Resumes test execution, so **should be used inside async function with `await`** operator.
    *
    * ```js
    * let tabs = await z.grabNumberOfOpenTabs();
    * ```
    *
    */
    static grabNumberOfOpenTabs() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let tabs = yield I.grabNumberOfOpenTabs();
                return tabs;
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield I.saveScreenshot(timestamp.toString() + ".png");
                yield this.generateLog(timestamp, error);
                throw new Error("grabNumberOfOpenTabs : Error fetching number of open tabs.");
            }
        });
    }
    /**
     * Checks that current url contains a provided fragment.
     *
     * @param {string} url value to check.
     *
     * ```js
     * await z.seeInCurrentUrl("url");
     * ```
     *
     */
    static seeInCurrentUrl(url) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield I.seeInCurrentUrl(url);
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield I.saveScreenshot(timestamp.toString() + ".png");
                yield this.generateLog(timestamp, error);
                throw new Error("seeInCurrentUrl : Error while seeing provided fragment " + url + " in current url");
            }
        });
    }
    /**
     * Retrieves a page title and returns it to test.
     * Resumes test execution, so **should be used inside async function with `await`** operator.
     *
     * ```js
     * let tabs = await z.grabTitle();
     * ```
     *
     */
    static grabTitle() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let title = yield I.grabTitle();
                return title;
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield I.saveScreenshot(timestamp.toString() + ".png");
                yield this.generateLog(timestamp, error);
                throw new Error("grabTitle : Error fetching title.");
            }
        });
    }
    /**
     * Scrolls element into viewport, waits for that element to become visible.
     * Then Verifies that the specified checkbox is not checked.
     *
     * ```js
     * z.dontSeeCheckboxIsChecked('#agree');
     * z.dontSeeCheckboxIsChecked('I agree to terms');
     * z.dontSeeCheckboxIsChecked('agree');
     * ```
     * @param {string} field  located by label|name|CSS|XPath|strict locator.
     */
    static dontSeeCheckboxIsChecked(field) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (field.startsWith("/") || field.startsWith(".") || field.startsWith("#") || field.startsWith("{") || field.startsWith("(")) {
                    yield I.waitForElement(field);
                    yield I.scrollIntoView(field, { behavior: "auto", block: "center", inline: "center" });
                    yield I.dontSeeCheckboxIsChecked(field);
                }
                else {
                    yield I.dontSeeCheckboxIsChecked(field);
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (field.startsWith("/") || field.startsWith(".") || field.startsWith("#") || field.startsWith("{") || field.startsWith("(")) {
                        yield I.waitForElement(field);
                        yield I.scrollIntoView(field, { behavior: "auto", block: "center", inline: "center" });
                        yield I.dontSeeCheckboxIsChecked(field);
                    }
                    else {
                        yield I.dontSeeCheckboxIsChecked(field);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("dontSeeCheckboxIsChecked : Error while verfiying checkbox is not checked: " + field);
                }
            }
        });
    }
    /**
     * Scrolls element into viewport, waits for that element to become visible.
     * Then Checks that the given input field or textarea equals to given value.
     *
     * ```js
     * z.seeInField('Username', 'davert');
     * z.seeInField({css: 'form textarea'},'Type your comment here');
     * z.seeInField('#searchform input','Search');
     * ```
     * @param {string} field located by label|name|CSS|XPath|strict locator.
     * @param {string} value value to check.
     */
    static seeInField(field, value) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (field.startsWith("/") || field.startsWith(".") || field.startsWith("#") || field.startsWith("{") || field.startsWith("(")) {
                    yield I.waitForElement(field);
                    yield I.scrollIntoView(field, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForVisible(field);
                    yield I.seeInField(field, value);
                }
                else {
                    yield I.seeInField(field, value);
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (field.startsWith("/") || field.startsWith(".") || field.startsWith("#") || field.startsWith("{") || field.startsWith("(")) {
                        yield I.waitForElement(field);
                        yield I.scrollIntoView(field, { behavior: "auto", block: "center", inline: "center" });
                        yield I.waitForVisible(field);
                        yield I.seeInField(field, value);
                    }
                    else {
                        yield I.seeInField(field, value);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("seeInField : Error while verfiying value in input field " + field);
                }
            }
        });
    }
    /**
    * Asserts that two values are not equal. If they are ,an assertion error is thrown.
    * ```js
    * z.assertNotEqual(expected,actual)
    * ```
    * @param {any} expected
    * @param {any} actual
    */
    static assertNotEqual(expected, actual, msg) {
        return __awaiter(this, void 0, void 0, function* () {
            yield I.assertNotEqual(expected, actual, msg);
        });
    }
    /**
    * Scrolls to element matched by locator. Extra shift can be set with offsetX and offsetY options.
    * ```js
    * z.scrollTo('footer');
    * z.scrollTo('#submit', 5, 5);
    
    * ```
    * @param {string} locator located by CSS|XPath|strict locator.
    * @param {number} offsetX (optional, 0 by default) X-axis offset
    * @param {number} offsetY (optional, 0 by default) Y-axis offset.
    *
    
    */
    static scrollTo(locator, offsetX, offsetY) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!util.isNullOrUndefined(offsetX)) {
                    yield I.scrollTo(locator, offsetX);
                }
                else if (!util.isNullOrUndefined(offsetY)) {
                    yield I.scrollTo(locator, undefined, offsetY);
                }
                else if (!util.isNullOrUndefined(offsetX) && !util.isNullOrUndefined(offsetY)) {
                    yield I.scrollTo(locator, offsetX, offsetY);
                }
                else {
                    yield I.scrollTo(locator);
                }
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield I.saveScreenshot(timestamp.toString() + ".png");
                yield this.generateLog(timestamp, error);
                throw new Error("scrollTo : Error while scroll to element located by: " + locator);
            }
        });
    }
    /**
     * Scroll page to the top.
     * ```js
     * z.scrollPageToTop();
     
     */
    static scrollPageToTop() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield I.scrollPageToTop();
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield I.saveScreenshot(timestamp.toString() + ".png");
                yield this.generateLog(timestamp, error);
                throw new Error("scrollPageToTop : Error while scroll page to top ");
            }
        });
    }
    /**
     * Scroll page to the bottom.
     * ```js
     * z.scrollPageToBottom();
     
     */
    static scrollPageToBottom() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield I.scrollPageToBottom();
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield I.saveScreenshot(timestamp.toString() + ".png");
                yield this.generateLog(timestamp, error);
                throw new Error("scrollPageToBottom : Error while scroll page to bottom ");
            }
        });
    }
    /**
    * Asserts that an element is visible a given number of times
    * ```js
    * await z..seeNumberOfVisibleElements('.buttons', 3);
    * ```
    * @param {string} locator CSS|XPath|strict locator.
    * @param {number} num number of elements..
    *
    */
    static seeNumberOfVisibleElements(locator, num) {
        return __awaiter(this, void 0, void 0, function* () {
            yield I.seeNumberOfVisibleElements(locator, num);
        });
    }
    /**
     * Scrolls element into viewport, waits for that element to become visible.
     * Then Verifies that the specified checkbox is checked.
     *
     * ```js
     * z.seeCheckboxIsChecked('Agree');
     * z.seeCheckboxIsChecked('#agree');
     * z.seeCheckboxIsChecked({css: '#signup_form input[type=checkbox]'});
     * ```
     * @param {string} field  located by label|name|CSS|XPath|strict locator.
     */
    static seeCheckboxIsChecked(field) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (field.startsWith("/") || field.startsWith(".") || field.startsWith("#") || field.startsWith("{") || field.startsWith("(")) {
                    yield I.waitForElement(field);
                    yield I.scrollIntoView(field, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForVisible(field);
                    yield I.seeCheckboxIsChecked(field);
                }
                else {
                    yield I.seeCheckboxIsChecked(field);
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (field.startsWith("/") || field.startsWith(".") || field.startsWith("#") || field.startsWith("{") || field.startsWith("(")) {
                        yield I.waitForElement(field);
                        yield I.scrollIntoView(field, { behavior: "auto", block: "center", inline: "center" });
                        yield I.waitForVisible(field);
                        yield I.seeCheckboxIsChecked(field);
                    }
                    else {
                        yield I.seeCheckboxIsChecked(field);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("seeCheckboxIsChecked : Error while verfiying checkbox is checked: " + field);
                }
            }
        });
    }
    /**
    * Resize the current window to provided width and height.
    * First parameter can be set to `maximize`.
    * ```js
    * z.resizeWindow(width, height)
    
    */
    static resizeWindow(width, height) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!util.isNullOrUndefined(height)) {
                    yield I.resizeWindow(width, height);
                }
                else {
                    yield I.resizeWindow(width, 0);
                }
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield I.saveScreenshot(timestamp.toString() + ".png");
                yield this.generateLog(timestamp, error);
                throw new Error("resizeWindow : Error resizing the current window ");
            }
        });
    }
    /**
     * Types out the given text into an active field.
     * To slow down typing use a second parameter, to set interval between key presses.
     * Note: Should be used when fillField is not an option.
     * ```js
     * z.type('Type this out.');
     *
     * // typing values with a 100ms interval
     * z.type('4141555311111111', 100);
     * // When passing in an array
     * z.type(['T', 'E', 'X', 'T'])
     *
     * @param {string} key|keys  key or array of keys to type.
     * @param {number} delay (optional) delay in ms between key presses
     */
    static type(key, delay) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (util.isNullOrUndefined(delay)) {
                    yield I.type(key);
                }
                else {
                    yield I.type(key, delay);
                }
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield I.saveScreenshot(timestamp.toString() + ".png");
                yield this.generateLog(timestamp, error);
                throw new Error("ERROR" + error + "\n" + key);
            }
        });
    }
    /**
     * Performs a swipe Right inside an element.
     * Can be slow or fast swipe.
     * ```js
     * z.swipeRight('#container');
     *
     * @param {string} locator an element on which to perform swipe
     * @param {string}[speed='slow'] a speed to perform: `slow` or `fast`.
     */
    static swipeRight(locator, speed) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield I.swipeRight(locator, speed);
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield I.saveScreenshot(timestamp.toString() + ".png");
                yield this.generateLog(timestamp, error);
                throw new Error("swipeRight : Error performing swipe right inside an element ");
            }
        });
    }
    /**
     * Checks that title contains text.
     * ```js
     * z.seeInTitle('Home Page');
     * ```
     * @param {string} text value to check.
     */
    static seeInTitle(text) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield I.seeInTitle(text);
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("seeInTitle : Error while checking title contains text: " + text);
                }
            }
        });
    }
    /**
     * Scrolls element into viewport, waits for that element to become visible & enabled, then Force clicks an element
     
     * ```js
     * z.forceClick('#hiddenButton');
     * z.forceClick('Click me', '#hidden');
     * ```
    * @param {string} locator CSS|XPath|strict locator.
     * @param {string} context (optional, null by default) element to search in CSS|XPath|Strict locator.
     */
    static forceClick(locator, context) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                    yield I.waitForElement(locator);
                    yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForVisible(locator);
                    yield I.waitForEnabled(locator);
                    if (util.isNullOrUndefined(context)) {
                        yield I.forceClick(locator);
                    }
                    else {
                        yield I.forceClick(locator, context);
                    }
                }
                else {
                    if (util.isNullOrUndefined(context)) {
                        yield I.forceClick(locator);
                    }
                    else {
                        yield I.forceClick(locator, context);
                    }
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                        yield I.waitForElement(locator);
                        yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                        yield I.waitForVisible(locator);
                        yield I.waitForEnabled(locator);
                        if (util.isNullOrUndefined(context)) {
                            yield I.forceClick(locator);
                        }
                        else {
                            yield I.forceClick(locator, context);
                        }
                    }
                    else {
                        if (util.isNullOrUndefined(context)) {
                            yield I.forceClick(locator);
                        }
                        else {
                            yield I.forceClick(locator, context);
                        }
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("forceClick : Error while clicking element located by : " + locator);
                }
            }
        });
    }
    /**
     * Enters a directory In local filesystem.
     * Starts from a current directory
     * z.amInPath('output/downloads');
     * @param {string} openPath
     */
    static amInPath(openPath) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield I.amInPath(openPath);
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield this.generateLog(timestamp, error);
                yield I.saveScreenshot(timestamp.toString() + ".png");
                throw new Error("amInPath: Error while entering a directory in filesysytem " + openPath);
            }
        });
    }
    /**
     * Waits for file to be present in current directory.
     *
     * ```js
     * z.handleDownloads();
     * z.click('Download large File');
     * z.amInPath('output/downloads');
     * z.waitForFile('largeFilesName.txt', 10); // wait 10 seconds for file
     * ```
     * @param {string} name
     * @param {number} [sec] seconds to wait
     */
    static waitForFile(name, sec) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!util.isNullOrUndefined(sec)) {
                    yield I.waitForFile(name, sec);
                }
                else {
                    yield I.waitForFile(name);
                }
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield this.generateLog(timestamp, error);
                yield I.saveScreenshot(timestamp.toString() + ".png");
                throw new Error("waitForFile: Error while waiting for file to be present in current directory. " + name);
            }
        });
    }
    /**
     * Checks that file with a name including given text exists in the current directory.
     *
     *```js
    * z.handleDownloads();
    * z.click('Download as PDF');
    * z.amInPath('output/downloads');
    * z.seeFileNameMatching('.pdf');
    * ```
    */
    static seeFileNameMatching(text) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield I.wait(2);
                yield I.seeFileNameMatching(text);
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield this.generateLog(timestamp, error);
                yield I.saveScreenshot(timestamp.toString() + ".png");
                try {
                    yield I.wait(2);
                    yield I.seeFileNameMatching(text);
                }
                catch (error) {
                    throw new Error("seeFileNameMatching: Error while checking that file with a name including given text exists in the current directory. " + text);
                }
            }
        });
    }
    /**
     * Checks that file found by `seeFile` doesn't include text.
     * @param {string} text
     * @param {string} [encoding='utf8']
     */
    static dontSeeInThisFile(text, encoding) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (util.isNullOrUndefined(encoding)) {
                    yield I.dontSeeInThisFile(text);
                }
                else {
                    yield I.dontSeeInThisFile(text, encoding);
                }
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield this.generateLog(timestamp, error);
                yield I.saveScreenshot(timestamp.toString() + ".png");
                try {
                    if (util.isNullOrUndefined(encoding)) {
                        yield I.dontSeeInThisFile(text);
                    }
                    else {
                        yield I.dontSeeInThisFile(text, encoding);
                    }
                }
                catch (error) {
                    throw new Error("dontSeeInThisFile: Error while checking that file found doesn't include text. " + text);
                }
            }
        });
    }
    /**
    * Checks that all elements with given locator have given attributes.
    *
    * ```js
    * z.seeAttributesOnElements('//form', { method: "post"});
    * ```
    *
    * @param {CodeceptJS.LocatorOrString} locator located by CSS|XPath|strict locator.
    * @param {object} attributes attributes and their values to check.
    *
    */
    static seeAttributesOnElements(locator, attributes) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield I.wait(2);
                yield I.seeAttributesOnElements(locator, attributes);
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield this.generateLog(timestamp, error);
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    yield I.wait(2);
                    yield I.seeAttributesOnElements(locator, attributes);
                }
                catch (error) {
                    throw new Error("seeAttributesOnElements: Error while checking that all elements with given locator have given attributes. " + locator);
                }
            }
        });
    }
    /**
     * Checks that a given Element is present in the DOM
     * Element is located by CSS or XPath.
     *
     * ```js
     * z.seeElementInDOM('#modal');
     * ```
     * @param {string} locator element located by CSS|XPath|strict locator.
     *
     */
    static seeElementInDOM(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield I.seeElementInDOM(locator);
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield this.generateLog(timestamp, error);
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    yield I.seeElementInDOM(locator);
                }
                catch (error) {
                    throw new Error("seeElementInDOM: Error while checking that a given Element is present in the DOM. " + locator);
                }
            }
        });
    }
    /**
    * Appends text into the field.
    * A field can be located by text, accessibility id, id.
    *
    * ```js
    * z.appendField('name', 'davert');
    * ```
    *
    * @param {string} field
    * @param {string} value
    */
    static appendField(locator, value) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.click(locator);
                yield I.appendField(locator, value);
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    this.click(locator);
                    yield I.appendField(locator, value);
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("appendField: Error while appending the text into the field.. " + locator);
                }
            }
        });
    }
    /**
    * Waits for an element to become not attached to the DOM on a page (by default waits for 1sec).
    * Element can be located by CSS or XPath.
    *
    * ```js
    * z.waitForDetached('#popup');
    * ```
    *
    * @param {string} locator element located by CSS|XPath|strict locator.
    * @param {number} timeout (optional, `1` by default) time in seconds to wait
    */
    static waitForDetached(locator, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!util.isNullOrUndefined(timeout)) {
                    yield I.waitForDetached(locator, timeout);
                }
                else {
                    yield I.waitForDetached(locator);
                }
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield this.generateLog(timestamp, error);
                yield I.saveScreenshot(timestamp.toString() + ".png");
                throw new Error("waitForDetached : Error while waiting for element to become not attached to the DOM located by : " + locator);
            }
        });
    }
    /**
     * Sends POST request to API.
     *
     * ```js
     * I.sendPostRequest('/api/users.json', { "email": "user@user.com" });
     * ```
     *
     * @param {*} url
     * @param {*} payload
     * @param {object} headers
     */
    static sendPostRequest(url, payload, headers) {
        return __awaiter(this, void 0, void 0, function* () {
            let response;
            try {
                response = yield I.sendPostRequest(url, payload, headers);
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    response = yield I.sendPostRequest(url, payload, headers);
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("sendPostRequest: Error while sending POST request to API.");
                }
            }
            return response;
        });
    }
    /**
     * Send GET request to REST API
     *
     * ```js
     * I.sendGetRequest('/api/users.json');
     * ```
     *
     * @param {*} url
     * @param {object} headers
     */
    static sendGetRequest(url, headers) {
        return __awaiter(this, void 0, void 0, function* () {
            let response;
            try {
                response = yield I.sendGetRequest(url, headers);
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    response = yield I.sendGetRequest(url, headers);
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("sendGetRequest: Error while sending GET request to REST API");
                }
            }
            return response;
        });
    }
    /**
    * Opposite to `seeElementInDOM`. Checks that element is not on page.
    *
    * ```js
    * I.dontSeeElementInDOM('.nav'); // checks that element is not on page visible or not
    * ```
    *
    * @param {string} locator located by CSS|XPath|Strict locator.
    */
    static dontSeeElementInDOM(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield I.dontSeeElementInDOM(locator);
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    yield I.dontSeeElementInDOM(locator);
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("dontSeeElementInDOM: Error while checking that element is not on page");
                }
            }
        });
    }
    /**
     * Checks that text is equal to provided one.
     *
     * ```js
     * I.seeTextEquals('text', 'h1');
     * ```
     *
     * @param {string} text element value to check.
     * @param {string} optional [context]  element located by CSS|XPath|strict locator.
     */
    static seeTextEquals(text, context) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (util.isNullOrUndefined(context)) {
                    yield I.wait(2);
                    yield I.seeTextEquals(text);
                }
                else {
                    yield I.waitForVisible(context);
                    yield I.seeTextEquals(text, context);
                }
            }
            catch (error) {
                try {
                    let timestamp = new Date().getTime();
                    yield I.saveScreenshot(timestamp.toString() + ".png");
                    if (util.isNullOrUndefined(context)) {
                        yield I.wait(2);
                        yield I.seeTextEquals(text);
                    }
                    else {
                        yield I.waitForVisible(context);
                        yield I.seeTextEquals(text, context);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("seeTextEquals: Error while checking that text is equal to provided one.");
                }
            }
        });
    }
    /**
     * Checks that title is equal to provided one.
     *
     * ```js
     * z.seeTitleEquals('Test title.');
     * ```
     *
     * @param {string} text value to check.
     */
    static seeTitleEquals(text) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield I.seeTitleEquals(text);
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield I.saveScreenshot(timestamp.toString() + ".png");
                yield this.generateLog(timestamp, error);
                throw new Error("seeTitleEquals: Error while checking that title is equal to provided one.");
            }
        });
    }
    /**
     * Clears a cookie by name,
     * if none provided clears all cookies.
     *
     * ```js
     * z.clearCookie();
     * z.clearCookie('test');
     * ```
     *
     * @param {string} (optional, `null` by default) cookie name
     *
     */
    static clearCookie(cookie) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (util.isNullOrUndefined(cookie)) {
                    yield I.clearCookie();
                }
                else {
                    yield I.clearCookie(cookie);
                }
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield I.saveScreenshot(timestamp.toString() + ".png");
                yield this.generateLog(timestamp, error);
                throw new Error("clearCookie: Error while clearing a cookie by name");
            }
        });
    }
    /**
      * Retrieves the innerHTML from an element located by CSS or XPath and returns it to test.
      * Resumes test execution, so **should be used inside async function with `await`** operator.
      * If more than one element is found - an array of HTMLs returned.
      *
      * ```js
      * let postHTML = await z.grabHTMLFrom('#post');
      * ```
      *
      * @param {CodeceptJS.LocatorOrString} element located by CSS|XPath|strict locator.
      * @returns {Promise<string>} HTML code for an element
      */
    static grabHTMLFrom(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            let postHTML;
            try {
                if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                    yield I.waitForElement(locator);
                    yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForVisible(locator);
                    postHTML = yield I.grabHTMLFrom(locator);
                }
                else {
                    postHTML = yield I.grabHTMLFrom(locator);
                }
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield I.saveScreenshot(timestamp.toString() + ".png");
                try {
                    if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                        yield I.waitForElement(locator);
                        yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                        yield I.waitForVisible(locator);
                        postHTML = yield I.grabHTMLFrom(locator);
                    }
                    else {
                        postHTML = yield I.grabHTMLFrom(locator);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("grabHTMLFrom: Error while retrieving the innerHTML from an element");
                }
            }
            return postHTML;
        });
    }
    /**
     * Retrieves all the innerHTML from elements located by CSS or XPath and returns it to test.
     * Resumes test execution, so should be used inside async function with await operator.
     *
     * ```js
     * let postHTML = await z.grabHTMLFromAll('#post');
     * ```
     *
     * @param {CodeceptJS.LocatorOrString} element located by CSS|XPath|strict locator.
     * @returns {Promise<string>} HTML code for an element
     */
    static grabHTMLFromAll(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            let postHTMLs = [];
            try {
                if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                    yield I.waitForElement(locator);
                    yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForVisible(locator);
                    postHTMLs = yield I.grabHTMLFromAll(locator);
                }
                else {
                    postHTMLs = yield I.grabHTMLFromAll(locator);
                }
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield I.saveScreenshot(timestamp.toString() + ".png");
                try {
                    if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                        yield I.waitForElement(locator);
                        yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                        yield I.waitForVisible(locator);
                        postHTMLs = yield I.grabHTMLFromAll(locator);
                    }
                    else {
                        postHTMLs = yield I.grabHTMLFromAll(locator);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("grabHTMLFromAll: Error while retrieving all the innerHTML from elements");
                }
            }
            return postHTMLs;
        });
    }
    /**
    * Retrieves the cssProperty from an element located by CSS or XPath and returns it to test.
    * Resumes test execution, so **should be used inside async function with `await`** operator.
    * If more than one element is found - an array of HTMLs returned.
    *
    * ```js
    * const value = await I.grabCssPropertyFrom('h3', 'font-weight');
    * ```
    *
    * @param {CodeceptJS.LocatorOrString} element located by CSS|XPath|strict locator.
    *  @param {CodeceptJS.String} cssProperty CSS property name.
    * @returns {Promise<string>} CSS value
    */
    static grabCssPropertyFrom(locator, cssProperty) {
        return __awaiter(this, void 0, void 0, function* () {
            let postHTML = '';
            try {
                if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                    yield I.waitForElement(locator);
                    yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForVisible(locator);
                    postHTML = yield I.grabCssPropertyFrom(locator, cssProperty);
                }
                else {
                    postHTML = yield I.grabCssPropertyFrom(locator, cssProperty);
                }
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield I.saveScreenshot(timestamp.toString() + ".png");
                try {
                    if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                        yield I.waitForElement(locator);
                        yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                        yield I.waitForVisible(locator);
                        postHTML = yield I.grabCssPropertyFrom(locator, cssProperty);
                    }
                    else {
                        postHTML = yield I.grabCssPropertyFrom(locator, cssProperty);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("grabCssPropertyFrom: Error while retrieving the css property from an element");
                }
            }
            return postHTML;
        });
    }
    static grabCssPropertyFromAll(locator, cssProperty) {
        return __awaiter(this, void 0, void 0, function* () {
            let postHTML = [];
            try {
                if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                    yield I.waitForElement(locator);
                    yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                    yield I.waitForVisible(locator);
                    postHTML = yield I.grabCssPropertyFromAll(locator, cssProperty);
                }
                else {
                    postHTML = yield I.grabCssPropertyFromAll(locator, cssProperty);
                }
            }
            catch (error) {
                let timestamp = new Date().getTime();
                yield I.saveScreenshot(timestamp.toString() + ".png");
                try {
                    if (locator.startsWith("/") || locator.startsWith(".") || locator.startsWith("#") || locator.startsWith("{") || locator.startsWith("(")) {
                        yield I.waitForElement(locator);
                        yield I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                        yield I.waitForVisible(locator);
                        postHTML = yield I.grabCssPropertyFromAll(locator, cssProperty);
                    }
                    else {
                        postHTML = yield I.grabCssPropertyFromAll(locator, cssProperty);
                    }
                    yield this.generateLog(timestamp, error);
                }
                catch (error) {
                    throw new Error("grabCssPropertyFrom: Error while retrieving the css property from an element");
                }
            }
            return postHTML;
        });
    }
    static generateLog(timestamp, error) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log("failed for " + error.stack);
            // await I.saveScreenshot(timestamp.toString() + ".png");
        });
    }
    /**
     * Checks if element is visible and returns true of false accordingly.
     *
     * ```js
     * const value = await z.checkIfVisible("//input[@id='123']");
     * ```
     *
     * @param {string|object} locator element located by CSS|XPath|strict locator.
     * @returns {boolean} boolean
     */
    static checkIfVisible(selector, ...options) {
        return __awaiter(this, void 0, void 0, function* () {
            let flag = false;
            try {
                if (yield I.grabNumberOfVisibleElements(selector)) {
                    flag = true;
                }
                else {
                    flag = false;
                }
            }
            catch (err) {
                console.log(err);
            }
            return flag;
        });
    }
    /**
     * Downloads the file from docker agent to code base agent.
     * We can also pass an expected file name to be set for the downloaded file.
     *
     * ```js
     * const fileName = await z.dockerFileDownload();
     * const fileName = await z.dockerFileDownload("test.pdf");
     * ```
     *
     * @returns {string} file path in the repo (./tempDownload/test.pdf)
     */
    static dockerFileDownload(expectedFileName) {
        return __awaiter(this, void 0, void 0, function* () {
            const config = require('codeceptjs').config.get();
            const ggrPort = config.helpers.WebDriver.port;
            const ggrHost = config.helpers.WebDriver.host;
            var sessions = this.prototype.helpers['WebDriver'].browser;
            console.log("Browser session id ", sessions.sessionId);
            const helper = this.prototype.helpers['REST'];
            //SELENOID_HOST - ggr machien ip and port
            let res = yield helper.sendGetRequest(`http://${ggrHost}:${ggrPort}/host/${sessions.sessionId}`);
            console.log(`Response from ggr machine api\n${JSON.stringify(res.data)}`);
            let ipAdress = res.data["Name"];
            let port = res.data["Port"];
            console.log("Selenoid IP address " + ipAdress);
            //ipAdress - selenoid machien ip and port
            let response = yield helper.sendGetRequest(`http://${ipAdress}:${port}/status`);
            console.log(`Response from status api\n${JSON.stringify(response.data)}`);
            let obj1 = response.data["browsers"]["chrome"];
            let sessionId = "";
            for (const [key, value] of Object.entries(obj1)) {
                if (Object.keys(value).length) {
                    console.log(`key found --> ${key}`);
                    let sessionArray = value["test"]["sessions"];
                    for (const element of sessionArray) {
                        sessionId = element["id"];
                        if (sessions.sessionId.includes(sessionId)) {
                            console.log(`id --> ${sessionId}`);
                            break;
                        }
                    }
                    break;
                }
            }
            console.log(`Session id retrieved\n${sessionId}`);
            let secresponse = yield helper.sendGetRequest(`http://${ipAdress}:${port}/download/${sessionId}`);
            console.log(`Response from session api\n${JSON.stringify(secresponse.data)}`);
            let defaultCxmlDocument = new DOMParser().parseFromString(secresponse.data, "application/xml");
            ///////////////To get latest fiel downloaded///////////
            let fileIndex = (defaultCxmlDocument.documentElement.getElementsByTagName("a").length - 1);
            let fileName = defaultCxmlDocument.documentElement.getElementsByTagName("a")[fileIndex].getAttribute("href");
            /////////// To get first file downloaded(we can pass index instead of zero)
            // let fileName = defaultCxmlDocument.documentElement.getElementsByTagName("a")[0].getAttribute("href");
            console.log(`File name\n${fileName}`);
            let newFileName = "./tempDownload/" + fileName;
            if (expectedFileName) {
                newFileName = "./tempDownload/" + expectedFileName;
            }
            let file = fs.createWriteStream(newFileName);
            // const file = fs.createWriteStream("./tempDownload/"+fileName); 
            yield new Promise(function (resolve, reject) {
                http.get(`http://${ipAdress}:${port}/download/${sessionId}/${fileName}`, (response) => {
                    response.pipe(file);
                    setTimeout(() => {
                        resolve(true);
                    }, 10000);
                });
            });
            return "./tempDownload/" + fileName;
        });
    }
    /**
     * Starts the browser explicitly.
     *
     * ```js
     * await z.startBrowser();
     * ```
     */
    static startBrowser() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                console.log("========================>Start the Browser");
                yield this.prototype.helpers["WebDriver"]._startBrowser();
            }
            catch (err) {
                console.log("========================>ERROR" + err);
                console.log("========================>Starting the Browser Failed...");
            }
        });
    }
    /**
     * Stops the browser explicitly.
     *
     * ```js
     * await z.stopBrowser();
     * ```
     */
    static stopBrowser() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                console.log("========================>Stop the Browser");
                yield this.prototype.helpers["WebDriver"]._stopBrowser();
            }
            catch (err) {
                console.log("========================>ERROR" + err);
                console.log("========================>Stopping the Browser Failed...");
            }
        });
    }
    static grabPropertyOrAttributeValue(locator, attribute, elementLocatorStrategy = 'xpath') {
        return __awaiter(this, void 0, void 0, function* () {
            let value = '';
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const browser = yield this.prototype.helpers['WebDriver'].browser;
                    const elementRef = yield browser.findElement(elementLocatorStrategy, locator);
                    Object.keys(elementRef)
                        .forEach(function eachKey(key) {
                        return __awaiter(this, void 0, void 0, function* () {
                            value = yield browser.getElementAttribute(elementRef[key], attribute);
                            if (value == null) {
                                value = yield browser.getElementProperty(elementRef[key], attribute);
                            }
                            resolve(value);
                        });
                    });
                }
                catch (error) {
                    throw new Error('Error: Grab Property and attribute is throwing error for locator : ' + locator + 'while fetching attribute or property : ' + attribute);
                }
            }));
        });
    }
    static grabPropertyOrAttributeValueFromAll(locator, attribute, elementLocatorStrategy = 'xpath') {
        return __awaiter(this, void 0, void 0, function* () {
            let value;
            let valueArray = [];
            let count = 0;
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const browser = yield this.prototype.helpers['WebDriver'].browser;
                    const elementRef = yield browser.findElements(elementLocatorStrategy, locator);
                    for (const element in elementRef) {
                        Object.keys(elementRef[element])
                            .forEach(function eachKey(key) {
                            return __awaiter(this, void 0, void 0, function* () {
                                value = yield browser.getElementAttribute(elementRef[element][key], attribute);
                                if (value == null) {
                                    value = yield browser.getElementProperty(elementRef[element][key], attribute);
                                }
                                count = count + 1;
                                valueArray.push(value);
                                if (count == elementRef.length) {
                                    resolve(valueArray);
                                }
                            });
                        });
                    }
                }
                catch (error) {
                    reject(error);
                    throw new Error('Error: Grab attribute from all is throwing error for locator : ' + locator + 'while fetching attribute or property : ' + attribute);
                }
            }));
        });
    }
}
exports.nonHeal = nonHeal;
