"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.z = void 0;
const autoHeal_1 = require("./autoHeal");
const nonHeal_1 = require("./nonHeal");
const util = require('util');
const { I } = inject();
class z extends Helper {
    /**
     * Scrolls element into viewport, waits for that element to become visible & clickable, then performs click action.
     * Throws error if any of the action fails in two attempts.
     * The second parameter is a context (CSS or XPath locator) to narrow the search.
     * ```js
     * // simple link
     * z.click('Logout');
     * // button of form
     * z.click('Submit');
     * // CSS button
     * z.click('#form input[type=submit]');
     * // XPath
     * z.click('//form/*[@type=submit]');
     * // link in context
     * z.click('Logout', '#nav');
     * // using strict locator
     * z.click({css: 'nav a.login'});
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {string} context (optional, null by default) element to search in CSS|XPath|Strict locator.
     */
    static click(locator, ElementContext) {
        return __awaiter(this, void 0, void 0, function* () {
            let context;
            if (Array.isArray(ElementContext))
                context = ElementContext[0];
            else
                context = ElementContext;
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("("))) {
                    yield autoHeal_1.autoHeal.click(locator[0], locator[1], 120, 0, context);
                }
                else
                    yield nonHeal_1.nonHeal.click(locator[0], context);
            }
            else
                yield nonHeal_1.nonHeal.click(locator, context);
        });
    }
    /**
     * Scrolls element into viewport, waits for that element to become visible & enabled, then clears field value.
     * Throws error if any of the action fails in two attempts.
     * ```js
     * z.clearField('Email');
     * z.clearField('user[email]');
     * z.clearField('#email');
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     */
    static clearField(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("("))) {
                    yield autoHeal_1.autoHeal.clearField(locator[0], locator[1]);
                }
                else
                    yield nonHeal_1.nonHeal.clearField(locator[0]);
            }
            else
                yield nonHeal_1.nonHeal.clearField(locator);
        });
    }
    /**
     * Waits for element to be visible, Clears element value & fills given value.
     * Throws error if any of the action fails in two attempts.
     * ```js
     * // by label
     * z.fillField('Email', 'hello@world.com');
     * // by name
     * z.fillField('password', secret('123456'));
     * // by CSS
     * z.fillField('form#login input[name=username]', 'John');
     * // or by strict locator
     * z.fillField({css: 'form#login input[name=username]'} 'John');
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {string} value
     */
    static fillField(locator, value) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("("))) {
                    yield autoHeal_1.autoHeal.fillField(locator[0], value, locator[1]);
                }
                else
                    yield nonHeal_1.nonHeal.fillField(locator[0], value);
            }
            else
                yield nonHeal_1.nonHeal.fillField(locator, value);
        });
    }
    /**
     * Checks for the visibility of text, if optional context parameter given then searches for the visibility of text on the locator element.
     * Throws error if any of the action fails in two attempts.
     * ```js
     * z.see('Welcome'); // text welcome on a page
     * z.see('Welcome', '.content'); // text inside .content div
     * z.see('Register', {css: 'form.register'}); // use strict locator
     * ```
     * @param {string} text
     * @param {string} context optional, element located by CSS|Xpath|strict locator in which to search for text.
     */
    static see(text, context) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(context))
                yield nonHeal_1.nonHeal.see(text, context[0]);
            else
                yield nonHeal_1.nonHeal.see(text, context);
        });
    }
    /**
     * Checks the visibilty of an element.
     * Throws error if any of the action fails in two attempts.
     * ```js
     * z.seeElement('#modal');
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     */
    static seeElement(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator))
                yield autoHeal_1.autoHeal.seeElement(locator[0], locator[1]);
            else
                yield nonHeal_1.nonHeal.seeElement(locator);
        });
    }
    /**
     * Opens a webpage in browser with given url.
     * ```js
     * z.amOnPage('/'); // opens main page of website
     * z.amOnPage('https://github.com'); // opens github
     * z.amOnPage('/login'); // opens a login page
     * ```
     * @param {string} url
     */
    static amOnPage(url) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.amOnPage(url);
        });
    }
    /**
     * Asserts that two values are equal. If they are not,an assertion error is thrown.
     * ```js
     * z.assertEqual(expected,actual)
     * ```
     * @param {any} expected
     * @param {any} actual
     */
    static assertEqual(expected, actual, msg) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.assertEqual(expected, actual, msg);
        });
    }
    /**
     * Returns all window handles. Useful for referencing a specific handle when calling I.switchToWindow(handle)
     * ```js
     * const windows = await I.grabAllWindowHandles();
     * ```
     * @returns {Array} windows
     */
    static grabAllWindowHandles() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield nonHeal_1.nonHeal.grabAllWindowHandles();
        });
    }
    /**
     * Returns current window handle
     * ```js
     * const window = await z.grabCurrentWindowHandle();
     * ```
     * @returns {any} window
     */
    static grabCurrentWindowHandle() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield nonHeal_1.nonHeal.grabCurrentWindowHandle();
        });
    }
    /**
     * Grabs the number of visible elements
     * ```js
     * let numOfElements = await z.grabNumberOfVisibleElements('p');
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @returns {any} totalElements
     */
    static grabNumberOfVisibleElements(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator))
                return yield nonHeal_1.nonHeal.grabNumberOfVisibleElements(locator[0]);
            else
                return yield nonHeal_1.nonHeal.grabNumberOfVisibleElements(locator);
        });
    }
    /**
     * Open new tab and switch to it.
     * ```js
     * z.openNewTab();
     * ```
     */
    static openNewTab() {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.openNewTab();
        });
    }
    /**
     * Presses a key or a key in combination with modifier keys in the browser (on a focused element).
     * ```js
    * z.pressKey('Backspace');
    * ```
    *
    * To press a key in combination with modifier keys, pass the sequence as an array. All modifier keys (`'Alt'`, `'Control'`, `'Meta'`, `'Shift'`) will be released afterwards.
    *
    * ```js
    * z.pressKey(['Control', 'Z']);
    * ```
    *
    * For specifying operation modifier key based on operating system it is suggested to use `'CommandOrControl'`.
    * This will press `'Command'` (also known as `'Meta'`) on macOS machines and `'Control'` on non-macOS machines.
    *
    * ```js
    * z.pressKey(['CommandOrControl', 'Z']);
    * ```
    *
    * Some of the supported key names are:
    * - `'AltLeft'` or `'Alt'`
    * - `'AltRight'`
    * - `'ArrowDown'`
    * - `'ArrowLeft'`
    * - `'ArrowRight'`
    * - `'ArrowUp'`
    * - `'Backspace'`
    * - `'Clear'`
    * - `'ControlLeft'` or `'Control'`
    * - `'ControlRight'`
    * - `'Command'`
    * - `'CommandOrControl'`
    * - `'Delete'`
    * - `'End'`
    * - `'Enter'`
    * - `'Escape'`
    * - `'F1'` to `'F12'`
    * - `'Home'`
    * - `'Insert'`
    * - `'MetaLeft'` or `'Meta'`
    * - `'MetaRight'`
    * - `'Numpad0'` to `'Numpad9'`
    * - `'NumpadAdd'`
    * - `'NumpadDecimal'`
    * - `'NumpadDivide'`
    * - `'NumpadMultiply'`
    * - `'NumpadSubtract'`
    * - `'PageDown'`
    * - `'PageUp'`
    * - `'Pause'`
    * - `'Return'`
    * - `'ShiftLeft'` or `'Shift'`
    * - `'ShiftRight'`
    * - `'Space'`
    * - `'Tab'`
     * @param {string|Array} key
     */
    static pressKey(key) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.pressKey(key);
        });
    }
    /**
     * Reload the current page.
     * ```js
     * z.refreshPage();
     * ```
     */
    static refreshPage() {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.refreshPage();
        });
    }
    /**
     * Saves a screenshot to ouput folder (set in codecept.json or codecept.conf.js).
     * Filename is relative to output folder. Optionally resize the window to the full available page scrollHeight and scrollWidth
     * to capture the entire page by passing true in as the second argument.
     * ```js
     * z.saveScreenshot('debug.png');
     * z.saveScreenshot('debug.png', true) //resizes to available scrollHeight and scrollWidth before taking screenshot
     * ```
     * @param {string} fileName
     * @param {boolean} fullPage optional
     */
    static saveScreenshot(fileName, fullPage) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.saveScreenshot(fileName, fullPage);
        });
    }
    /**
     * Switches frame or in case of null locator reverts to parent.
     * ```js
     * z.switchTo('iframe'); // switch to first iframe
     * z.switchTo(); // switch back to main page
     * ```
     * @param {string} locator optional, CSS|XPath|strict locator.
     */
    static switchTo(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator))
                yield nonHeal_1.nonHeal.switchTo(locator[0]);
            else
                yield nonHeal_1.nonHeal.switchTo(locator);
        });
    }
    /**
     * Switch focus to a particular tab by its number. It waits tabs loading and then switch tab.
     * ```js
     * z.switchToNextTab();
     * z.switchToNextTab(2);
     * ```
     * @param {number} tabNum optional, number of tabs to switch forward, default: 1.
     * @param {number} timeout optional, time in seconds to wait.
     */
    static switchToNextTab(tabNum, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.switchToNextTab(tabNum, timeout);
        });
    }
    /**
     * Switch to the window with a specified handle.
     * ```js
     * const windows = await z.grabAllWindowHandles();
     * //  do something
     * await z.switchToWindow( windows[0] );
     *
     * const window = await z.grabCurrentWindowHandle();
     * //  do something
     * await z.switchToWindow( window );
     * ```
     * @param {string} window
     */
    static switchToWindow(window) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.switchToWindow(window);
        });
    }
    /**
     * Pauses execution for a number of seconds.
     * ```js
     * z.wait(2); // wait 2 secs
     * ```
     * @param {number} timeout
     */
    static wait(timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.wait(timeout);
        });
    }
    /**
     * Waits for the visibility of element & for the specified value to be in value attribute.
     * ```js
     * z.waitForValue('//input', "GoodValue");
     * ```
     * @param {string} locator input field locator.
     * @param {string} value
     * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
     */
    static waitForValue(locator, value, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    yield autoHeal_1.autoHeal.waitForValue(locator[0], value, locator[1], timeout);
                else
                    yield nonHeal_1.nonHeal.waitForValue(locator[0], value, timeout);
            }
            else
                yield nonHeal_1.nonHeal.waitForValue(locator, value, timeout);
        });
    }
    /**
     * Waits for the visibility of element.
     * ```js
     * z.waitForVisible('#popup');
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
     */
    static waitForVisible(locator, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    yield autoHeal_1.autoHeal.waitForVisible(locator[0], locator[1], timeout);
                else
                    yield nonHeal_1.nonHeal.waitForVisible(locator[0], timeout);
            }
            else
                yield nonHeal_1.nonHeal.waitForVisible(locator, timeout);
        });
    }
    /**
     * Waits for a text to appear.
     * Element can be located by CSS or XPath. Narrow down search results by providing context.
     * ```js
     * z.waitForText('Thank you, form has been submitted');
     * z.waitForText('Thank you, form has been submitted', 5, '#modal');
     * ```
     * @param {string} text
     * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
     * @param {string} context optional, element located by CSS|XPath|strict locator.
     
     */
    static waitForText(text, timeout, context) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(context))
                yield nonHeal_1.nonHeal.waitForText(text, timeout, context[0]);
            else
                yield nonHeal_1.nonHeal.waitForText(text, timeout, context);
        });
    }
    /**
     * Waits for an element to be removed or become invisible on a page.
     * Element can be located by CSS or XPath.
     * ```js
     * z.waitForInvisible('#popup');
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
     */
    static waitForInvisible(locator, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    yield autoHeal_1.autoHeal.waitForInvisible(locator[0], locator[1], timeout);
                else
                    yield nonHeal_1.nonHeal.waitForInvisible(locator[0], timeout);
            }
            else
                yield nonHeal_1.nonHeal.waitForInvisible(locator, timeout);
        });
    }
    /**
     * Waits for an element to become enabled.
     * Element can be located by CSS or XPath.
     * ```js
     * z.waitForEnabled('#popup');
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
     */
    static waitForEnabled(locator, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    yield autoHeal_1.autoHeal.waitForEnabled(locator[0], locator[1], timeout);
                else
                    yield nonHeal_1.nonHeal.waitForEnabled(locator[0], timeout);
            }
            else
                yield nonHeal_1.nonHeal.waitForEnabled(locator, timeout);
        });
    }
    /**
     * Waits for an element to be present on page.
     * Element can be located by CSS or XPath.
     * ```js
     * z.waitForElement('.btn.continue');
     * z.waitForElement('.btn.continue', 5); // wait for 5 secs
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
     */
    static waitForElement(locator, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    yield autoHeal_1.autoHeal.waitForElement(locator[0], locator[1], timeout);
                else
                    yield nonHeal_1.nonHeal.waitForElement(locator[0], timeout);
            }
            else
                yield nonHeal_1.nonHeal.waitForElement(locator, timeout);
        });
    }
    /**
     * Waits for an element to be clickable.
     * Element can be located by CSS or XPath.
     * ```js
     * z.waitForClickable('.btn.continue');
     * z.waitForClickable('.btn.continue', 5); // wait for 5 secs
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
     */
    static waitForClickable(locator, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    yield autoHeal_1.autoHeal.waitForClickable(locator[0], locator[1], timeout);
                else
                    yield nonHeal_1.nonHeal.waitForClickable(locator[0], timeout);
            }
            else
                yield nonHeal_1.nonHeal.waitForClickable(locator, timeout);
        });
    }
    /**
     * Waits for an element to be visible.
     * Element can be located by CSS or XPath.
     * ```js
     * z.attachFile('Avatar', 'data/avatar.jpg');
     * z.attachFile('form input[name=avatar]', 'data/avatar.jpg');
     * ```
     * @param {string} locator label|name|CSS|XPath|strict locator.
     * @param {string} filepath local file path relative to codecept.json config file
     */
    static attachFile(locator, filepath) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    yield autoHeal_1.autoHeal.attachFile(locator[0], filepath, locator[1]);
                else
                    yield nonHeal_1.nonHeal.attachFile(locator[0], filepath);
            }
            else
                yield nonHeal_1.nonHeal.attachFile(locator, filepath);
        });
    }
    /**
     * Switch focus to a particular tab by its number. It waits tabs loading and then switch tab.
     * ```js
     * z.switchToPreviousTab();
     * z.switchToPreviousTab(2);
     * ```
     * @param {number} tabNum optional, number of tabs to switch backward, default: 1.
     * @param {number} timeout optional, time in seconds to wait.
     */
    static switchToPreviousTab(tabNum, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.switchToPreviousTab(tabNum, timeout);
        });
    }
    /**
     * Close current tab.
     * ```js
     * z.closeCurrentTab();
     * ```
     */
    static closeCurrentTab() {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.closeCurrentTab();
        });
    }
    /**
     * Close all tabs except for the current one.
     * ```js
     * z.closeOtherTabs();
     * ```
     */
    static closeOtherTabs() {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.closeOtherTabs();
        });
    }
    /**
     * Scroll element into viewport.
     * ```js
     * z.scrollIntoView('#submit');
     * z.scrollIntoView('#submit', { behavior: "smooth", block: "center", inline: "center" });
     * z.scrollIntoView('#submit', true);
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {any} alignTop optional, or scrollIntoViewOptions (optional),
     * see https://developer.mozilla.org/en-US/docs/Web/API/Element/scrollIntoView.
     
     */
    static scrollIntoView(locator, alignTop) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator))
                yield nonHeal_1.nonHeal.scrollIntoView(locator[0], alignTop);
            else
                yield nonHeal_1.nonHeal.scrollIntoView(locator, alignTop);
        });
    }
    /**
     * Retrieves a text from an element located by CSS or XPath and returns it to test.
     * Resumes test execution, so should be used inside static async with await operator.
     * If multiple elements found returns an array of texts.
     * ```js
     * let pin = await z.grabTextFrom('#pin');
     * ```
     * @param {string} locator element located by CSS|XPath|strict locator.
     * @returns {string|string[]} text
     */
    static grabTextFrom(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            let text = '';
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    text = yield autoHeal_1.autoHeal.grabTextFrom(locator[0], locator[1]);
                else
                    text = yield nonHeal_1.nonHeal.grabTextFrom(locator[0]);
            }
            else {
                text = yield nonHeal_1.nonHeal.grabTextFrom(locator);
            }
            return text;
        });
    }
    static grabTextFromAll(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            let text = [];
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    text = yield autoHeal_1.autoHeal.grabTextFromAll(locator[0], locator[1]);
                else
                    text = yield nonHeal_1.nonHeal.grabTextFromAll(locator[0]);
            }
            else
                text = yield nonHeal_1.nonHeal.grabTextFromAll(locator);
            return text;
        });
    }
    /**
     * Retrieves an attribute from an element located by CSS or XPath and returns it to test. An array as a result will be returned if there are more than one matched element.
     * Resumes test execution, so should be used inside static async with await operator.
     * ```js
     * let hint = await z.grabAttributeFrom('#tooltip', 'title');
     * ```
     * @param {string} locator element located by CSS|XPath|strict locator.
     * @param {string} attribute attribute name.
     * @returns {string} text
     */
    static grabAttributeFrom(locator, attribute) {
        return __awaiter(this, void 0, void 0, function* () {
            let value = "";
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    value = yield autoHeal_1.autoHeal.grabAttributeFrom(locator[0], attribute, locator[1]);
                else
                    value = yield nonHeal_1.nonHeal.grabAttributeFrom(locator[0], attribute);
            }
            else
                value = yield nonHeal_1.nonHeal.grabAttributeFrom(locator, attribute);
            return value;
        });
    }
    static grabAttributeFromAll(locator, attribute) {
        return __awaiter(this, void 0, void 0, function* () {
            let value = [];
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    value = yield autoHeal_1.autoHeal.grabAttributeFromAll(locator[0], attribute, locator[1]);
                else
                    value = yield nonHeal_1.nonHeal.grabAttributeFromAll(locator[0], attribute);
            }
            else
                value = yield nonHeal_1.nonHeal.grabAttributeFromAll(locator, attribute);
            return value;
        });
    }
    /**
         * Scrolls element into viewport, waits for that element to become visible & clickable. Also checks if element is already checked.
         * Then selects a checkbox or radio button.
         *
         * The second parameter is a context (CSS or XPath locator) to narrow the search.
         * ```js
         * z.checkOption('#agree');
         * z.checkOption('I Agree to Terms and Conditions');
         * z.checkOption('agree', '//form');
         * ```
         * @param {string} locator checkbox located by label | name | CSS | XPath | strict locator.
         * @param {string} context optional element located by CSS | XPath | strict locator.
         */
    static checkOption(locator, context) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    yield autoHeal_1.autoHeal.checkOption(locator[0], locator[1], 120, 0, context);
                else {
                    if (Array.isArray(context))
                        yield nonHeal_1.nonHeal.checkOption(locator[0], context[0]);
                    else
                        yield nonHeal_1.nonHeal.checkOption(locator[0], context);
                }
            }
            else {
                if (Array.isArray(context))
                    yield nonHeal_1.nonHeal.checkOption(locator, context[0]);
                else
                    yield nonHeal_1.nonHeal.checkOption(locator, context);
            }
        });
    }
    /**
     * Checks that a text is not present on a page, if optional context parameter given then searches for the invisibility of text on the locator element.
     * Use context parameter to narrow down the search.
     * Throws error if any of the action fails in two attempts.
     * ```js
     * z.dontSee('Login'); // assume we are already logged in.
     * z.dontSee('Login', '.nav'); // no login inside .nav element
     * ```
     * @param {string} text which is not present.
     * @param {string} context optional, element located by CSS|Xpath|strict locator in which to search for text.
     */
    static dontSee(text, context) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(context))
                yield nonHeal_1.nonHeal.dontSee(text, context[0]);
            else
                yield nonHeal_1.nonHeal.dontSee(text, context);
        });
    }
    /**
     * Checks that element is not visible (or in DOM)
     * Throws error if any of the action fails in two attempts.
     * ```js
     * z.seeElement('#modal');
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     */
    static dontSeeElement(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator))
                yield nonHeal_1.nonHeal.dontSeeElement(locator[0]);
            else
                yield nonHeal_1.nonHeal.dontSeeElement(locator);
        });
    }
    /**
     * Scrolls element into viewport, waits for that element to become visible & clickable, then performs double click action.
     * Throws error if any of the action fails in two attempts.
     * The second parameter is a context (CSS or XPath locator) to narrow the search.
     * ```js
     * z.doubleClick('Edit');
     * z.doubleClick('Edit', '.actions');
     * z.doubleClick({css: 'button.accept'});
     * z.doubleClick('.btn.edit');
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {string} context (optional, null by default) element to search in CSS|XPath|Strict locator.
     */
    static doubleClick(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    yield autoHeal_1.autoHeal.doubleClick(locator[0], locator[1]);
                else
                    yield nonHeal_1.nonHeal.doubleClick(locator[0]);
            }
            else
                yield nonHeal_1.nonHeal.doubleClick(locator);
        });
    }
    /**
     * Scrolls element into viewport, waits for that element to become visible & clickable, then Drag an item to a destination element.
     * Throws error if any of the action fails in two attempts.
     *
     * ```js
     * z.dragAndDrop('#dragHandle', '#container');
     * ```
     * @param {string} srcLocator CSS|XPath|strict locator.
     * @param {string} destLocator CSS|XPath|Strict locator.
     */
    static dragAndDrop(srcLocator, destLocator) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(srcLocator) && Array.isArray(destLocator))
                yield nonHeal_1.nonHeal.dragAndDrop(srcLocator[0], destLocator[0]);
            else if (Array.isArray(srcLocator) && !Array.isArray(destLocator))
                yield nonHeal_1.nonHeal.dragAndDrop(srcLocator[0], destLocator);
            else if (!Array.isArray(srcLocator) && Array.isArray(destLocator))
                yield nonHeal_1.nonHeal.dragAndDrop(srcLocator, destLocator[0]);
            else
                yield nonHeal_1.nonHeal.dragAndDrop(srcLocator, destLocator);
        });
    }
    /**
     * Executes sync script on a page.
     * Pass arguments to function as additional parameters.
     * Will return execution result to a test.
     * In this case you should use static async function and await to receive results.
     *
     * Example with jQuery DatePicker:
     *
     * ```js
     * // change date of jQuery DatePicker
     * z.executeScript(function() {
     *   // now we are inside browser context
     *   $('date').datetimepicker('setDate', new Date());
     * });
     * ```
     * Can return values. Don't forget to use `await` to get them.
     *
     * ```js
     * let date = await z.executeScript(function(el)
     *   // only basic types can be returned
     *   return $(el).datetimepicker('getDate').tostring();
     * } '#date'); // passing jquery selector
     * ```
     * @param {string|function} fn function to be executed in browser context.
     * @param {...any} args to be passed to function.
     * */
    static executeScript(fn, ...args) {
        return __awaiter(this, void 0, void 0, function* () {
            let retunrtype = yield I.executeScript(fn, args);
            return retunrtype;
        });
    }
    /**
     * Scrolls element into viewport, waits for that element to become visible & clickable, then selects an option in a drop-down select.
     * Throws error if any of the action fails in two attempts.
     *
     * ```js
     * z.selectOption('Choose Plan', 'Monthly'); // select by label
     * z.selectOption('subscription', 'Monthly'); // match option by text
     * z.selectOption('subscription', '0'); // or by value
     * z.selectOption('//form/select[@name=account]','Premium');
     * z.selectOption('form select[name=account]', 'Premium');
     * z.selectOption({css: 'form select[name=account]'} 'Premium');
     * ```
     * Provide an array for the second argument to select multiple options.
     * ```js
     * z.selectOption('Which OS do you use?', ['Android', 'iOS']);
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {string|string[]} option CSS|XPath|Strict locator.
     */
    static selectOption(locator, option) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    yield autoHeal_1.autoHeal.selectOption(locator[0], option, locator[1]);
                else
                    yield nonHeal_1.nonHeal.selectOption(locator[0], option);
            }
            else
                yield nonHeal_1.nonHeal.selectOption(locator, option);
        });
    }
    /**
     * Checks for element to be present, scrolls into view & Moves cursor to element matched by locator.
     * Extra shift can be set with offsetX and offsetY options.
     *
     * ```js
     * z.moveCursorTo('.tooltip');
     * z.moveCursorTo('#submit', 5,5);
     * ```
     *
     * @param {CodeceptJS.LocatorOrString} locator located by CSS|XPath|strict locator.
     * @param {number} [offsetX=0] (optional, `0` by default) X-axis offset.
     * @param {number} [offsetY=0] (optional, `0` by default) Y-axis offset.
     *
     *
     */
    static moveCursorTo(locator, offsetX, offsetY) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    yield autoHeal_1.autoHeal.moveCursorTo(locator[0], locator[1], offsetX, offsetY);
                else
                    yield nonHeal_1.nonHeal.moveCursorTo(locator[0], offsetX, offsetY);
            }
            else
                yield nonHeal_1.nonHeal.moveCursorTo(locator, offsetX, offsetY);
        });
    }
    /**
     * Checks for element to be present, scrolls into view, waits for element to be visible and retrieves a value from a form element located by CSS or XPath, returns it to test.
     * Resumes test execution, so **should be used inside async function with `await`** operator.
     *
     * ```js
     * let email = await z.grabValueFrom('input[name=email]');
     * ```
     * @param {CodeceptJS.LocatorOrString} locator field located by label|name|CSS|XPath|strict locator.
     * @returns {Promise<any>} attribute value
     *
     */
    static grabValueFrom(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            let text = "";
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    text = yield autoHeal_1.autoHeal.grabValueFrom(locator[0], locator[1]);
                else
                    text = yield nonHeal_1.nonHeal.grabValueFrom(locator[0]);
            }
            else
                text = yield nonHeal_1.nonHeal.grabValueFrom(locator);
            return text;
        });
    }
    static grabValueFromAll(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            let text;
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    text = yield autoHeal_1.autoHeal.grabValueFromAll(locator[0], locator[1]);
                else
                    text = yield nonHeal_1.nonHeal.grabValueFromAll(locator[0]);
            }
            else
                text = yield nonHeal_1.nonHeal.grabValueFromAll(locator);
            return text;
        });
    }
    /**
     * Dismisses the active JavaScript popup, as created by window.alert|window.confirm|window.prompt.
     * ```js
     * await z.cancelPopup();
     * ```
     */
    static cancelPopup() {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.cancelPopup();
        });
    }
    /**
     * Accepts the active JavaScript native popup window, as created by window.alert|window.confirm|window.prompt.
     * Don't confuse popups with modal windows, as created by [various
     * libraries](http://jster.net/category/windows-modals-popups).
     * ```js
     * await z.acceptPopup();
     * ```
     */
    static acceptPopup() {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.acceptPopup();
        });
    }
    /**
     * Checks that the active JavaScript popup, as created by `window.alert|window.confirm|window.prompt`, contains the
     * given string.
     *
     * @param {string} text value to check.
     *
     * ```js
     * await z.seeInPopup("value");
     * ```
     *
     */
    static seeInPopup(text) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.seeInPopup(text);
        });
    }
    /**
     * Gets a cookie object by name.
     * If none provided gets all cookies.
     * Resumes test execution, so **should be used inside async function with `await`** operator.
     *
     * ```js
     * let cookie = await z.grabCookie('auth');
     * assert(cookie.value, '123456');
     * ```
     *
     * @param {?string} [name=null] cookie name.
     * @returns {Promise<string>} attribute value
     *
     */
    static grabCookie(name) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield nonHeal_1.nonHeal.grabCookie(name);
        });
    }
    /**
     * Get current URL from browser
     * Resumes test execution, so **should be used inside async function with `await`** operator.
     *
     * ```js
     * let url = await z.grabCurrentUrl();
     * ```
     *
     */
    static grabCurrentUrl() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield nonHeal_1.nonHeal.grabCurrentUrl();
        });
    }
    /**
    * Grab number of open tabs.
    * Resumes test execution, so **should be used inside async function with `await`** operator.
    *
    * ```js
    * let tabs = await z.grabNumberOfOpenTabs();
    * ```
    *
    */
    static grabNumberOfOpenTabs() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield nonHeal_1.nonHeal.grabNumberOfOpenTabs();
        });
    }
    /**
     * Checks that current url contains a provided fragment.
     *
     * @param {string} url value to check.
     *
     * ```js
     * await z.seeInCurrentUrl("url");
     * ```
     *
     */
    static seeInCurrentUrl(url) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.seeInCurrentUrl(url);
        });
    }
    /**
     * Retrieves a page title and returns it to test.
     * Resumes test execution, so **should be used inside async function with `await`** operator.
     *
     * ```js
     * let tabs = await z.grabTitle();
     * ```
     *
     */
    static grabTitle() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield nonHeal_1.nonHeal.grabTitle();
        });
    }
    /**
     * Scrolls element into viewport, waits for that element to become visible.
     * Then Verifies that the specified checkbox is not checked.
     *
     * ```js
     * z.dontSeeCheckboxIsChecked('#agree');
     * z.dontSeeCheckboxIsChecked('I agree to terms');
     * z.dontSeeCheckboxIsChecked('agree');
     * ```
     * @param {string} field  located by label|name|CSS|XPath|strict locator.
     */
    static dontSeeCheckboxIsChecked(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    yield autoHeal_1.autoHeal.dontSeeCheckboxIsChecked(locator[0], locator[1]);
                else
                    yield nonHeal_1.nonHeal.dontSeeCheckboxIsChecked(locator[0]);
            }
            else
                yield nonHeal_1.nonHeal.dontSeeCheckboxIsChecked(locator);
        });
    }
    /**
     * Scrolls element into viewport, waits for that element to become visible.
     * Then Checks that the given input field or textarea equals to given value.
     *
     * ```js
     * z.seeInField('Username', 'davert');
     * z.seeInField({css: 'form textarea'},'Type your comment here');
     * z.seeInField('#searchform input','Search');
     * ```
     * @param {string} field located by label|name|CSS|XPath|strict locator.
     * @param {string} value value to check.
     */
    static seeInField(locator, value) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    yield autoHeal_1.autoHeal.seeInField(locator[0], value, locator[1]);
                else
                    yield nonHeal_1.nonHeal.seeInField(locator[0], value);
            }
            else
                yield nonHeal_1.nonHeal.seeInField(locator, value);
        });
    }
    /**
    * Asserts that two values are not equal. If they are ,an assertion error is thrown.
    * ```js
    * z.assertNotEqual(expected,actual)
    * ```
    * @param {any} expected
    * @param {any} actual
    */
    static assertNotEqual(expected, actual, msg) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.assertNotEqual(expected, actual, msg);
        });
    }
    /**
    * Scrolls to element matched by locator. Extra shift can be set with offsetX and offsetY options.
    * ```js
    * z.scrollTo('footer');
    * z.scrollTo('#submit', 5, 5);
    
    * ```
    * @param {string} locator located by CSS|XPath|strict locator.
    * @param {number} offsetX (optional, 0 by default) X-axis offset
    * @param {number} offsetY (optional, 0 by default) Y-axis offset.
    *
    
    */
    static scrollTo(locator, offsetX, offsetY) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator))
                yield nonHeal_1.nonHeal.scrollTo(locator[0], offsetX, offsetY);
            else
                yield nonHeal_1.nonHeal.scrollTo(locator, offsetX, offsetY);
        });
    }
    /**
     * Scroll page to the top.
     * ```js
     * z.scrollPageToTop();
     
     */
    static scrollPageToTop() {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.scrollPageToTop();
        });
    }
    /**
     * Scroll page to the bottom.
     * ```js
     * z.scrollPageToBottom();
     
     */
    static scrollPageToBottom() {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.scrollPageToBottom();
        });
    }
    /**
    * Asserts that an element is visible a given number of times
    * ```js
    * await z..seeNumberOfVisibleElements('.buttons', 3);
    * ```
    * @param {string} locator CSS|XPath|strict locator.
    * @param {number} num number of elements..
    *
    */
    static seeNumberOfVisibleElements(locator, num) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator))
                yield nonHeal_1.nonHeal.seeNumberOfVisibleElements(locator[0], num);
            else
                yield nonHeal_1.nonHeal.seeNumberOfVisibleElements(locator, num);
        });
    }
    /**
     * Scrolls element into viewport, waits for that element to become visible.
     * Then Verifies that the specified checkbox is checked.
     *
     * ```js
     * z.seeCheckboxIsChecked('Agree');
     * z.seeCheckboxIsChecked('#agree');
     * z.seeCheckboxIsChecked({css: '#signup_form input[type=checkbox]'});
     * ```
     * @param {string} field  located by label|name|CSS|XPath|strict locator.
     */
    static seeCheckboxIsChecked(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    yield autoHeal_1.autoHeal.seeCheckboxIsChecked(locator[0], locator[1]);
                else
                    yield nonHeal_1.nonHeal.seeCheckboxIsChecked(locator[0]);
            }
            else
                yield nonHeal_1.nonHeal.seeCheckboxIsChecked(locator);
        });
    }
    /**
    * Resize the current window to provided width and height.
    * First parameter can be set to `maximize`.
    * ```js
    * z.resizeWindow(width, height)
    
    */
    static resizeWindow(width, height) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.resizeWindow(width, height);
        });
    }
    /**
     * Types out the given text into an active field.
     * To slow down typing use a second parameter, to set interval between key presses.
     * Note: Should be used when fillField is not an option.
     * ```js
     * z.type('Type this out.');
     *
     * // typing values with a 100ms interval
     * z.type('4141555311111111', 100);
     * // When passing in an array
     * z.type(['T', 'E', 'X', 'T'])
     *
     * @param {string} key|keys  key or array of keys to type.
     * @param {number} delay (optional) delay in ms between key presses
     */
    static type(key, delay) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.type(key, delay);
        });
    }
    /**
     * Performs a swipe Right inside an element.
     * Can be slow or fast swipe.
     * ```js
     * z.swipeRight('#container');
     *
     * @param {string} locator an element on which to perform swipe
     * @param {string}[speed='slow'] a speed to perform: `slow` or `fast`.
     */
    static swipeRight(locator, speed) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator))
                yield nonHeal_1.nonHeal.swipeRight(locator[0], speed);
            else
                yield nonHeal_1.nonHeal.swipeRight(locator, speed);
        });
    }
    /**
     * Checks that title contains text.
     * ```js
     * z.seeInTitle('Home Page');
     * ```
     * @param {string} text value to check.
     */
    static seeInTitle(text) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.seeInTitle(text);
        });
    }
    /**
     * Scrolls element into viewport, waits for that element to become visible & enabled, then Force clicks an element
     
     * ```js
     * z.forceClick('#hiddenButton');
     * z.forceClick('Click me', '#hidden');
     * ```
    * @param {string} locator CSS|XPath|strict locator.
     * @param {string} context (optional, null by default) element to search in CSS|XPath|Strict locator.
     */
    static forceClick(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    yield autoHeal_1.autoHeal.forceClick(locator[0], locator[1]);
                else
                    yield nonHeal_1.nonHeal.forceClick(locator[0]);
            }
            else
                yield nonHeal_1.nonHeal.forceClick(locator);
        });
    }
    /**
     * Enters a directory In local filesystem.
     * Starts from a current directory
     * z.amInPath('output/downloads');
     * @param {string} openPath
     */
    static amInPath(openPath) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.amInPath(openPath);
        });
    }
    /**
     * Waits for file to be present in current directory.
     *
     * ```js
     * z.handleDownloads();
     * z.click('Download large File');
     * z.amInPath('output/downloads');
     * z.waitForFile('largeFilesName.txt', 10); // wait 10 seconds for file
     * ```
     * @param {string} name
     * @param {number} [sec] seconds to wait
     */
    static waitForFile(name, sec) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.waitForFile(name, sec);
        });
    }
    /**
     * Checks that file with a name including given text exists in the current directory.
     *
     *```js
    * z.handleDownloads();
    * z.click('Download as PDF');
    * z.amInPath('output/downloads');
    * z.seeFileNameMatching('.pdf');
    * ```
    */
    static seeFileNameMatching(text) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.seeFileNameMatching(text);
        });
    }
    /**
     * Checks that file found by `seeFile` doesn't include text.
     * @param {string} text
     * @param {string} [encoding='utf8']
     */
    static dontSeeInThisFile(text, encoding) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.dontSeeInThisFile(text, encoding);
        });
    }
    /**
    * Checks that all elements with given locator have given attributes.
    *
    * ```js
    * z.seeAttributesOnElements('//form', { method: "post"});
    * ```
    *
    * @param {CodeceptJS.LocatorOrString} locator located by CSS|XPath|strict locator.
    * @param {object} attributes attributes and their values to check.
    *
    */
    static seeAttributesOnElements(locator, attributes) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    yield autoHeal_1.autoHeal.seeAttributesOnElements(locator[0], attributes, locator[1]);
                else
                    yield nonHeal_1.nonHeal.seeAttributesOnElements(locator[0], attributes);
            }
            else
                yield nonHeal_1.nonHeal.seeAttributesOnElements(locator, attributes);
        });
    }
    /**
     * Checks that a given Element is present in the DOM
     * Element is located by CSS or XPath.
     *
     * ```js
     * z.seeElementInDOM('#modal');
     * ```
     * @param {string} locator element located by CSS|XPath|strict locator.
     *
     */
    static seeElementInDOM(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator))
                yield nonHeal_1.nonHeal.seeElementInDOM(locator[0]);
            else
                yield nonHeal_1.nonHeal.seeElementInDOM(locator);
        });
    }
    /**
    * Appends text into the field.
    * A field can be located by text, accessibility id, id.
    *
    * ```js
    * z.appendField('name', 'davert');
    * ```
    *
    * @param {string} field
    * @param {string} value
    */
    static appendField(locator, value) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    yield autoHeal_1.autoHeal.appendField(locator[0], value, locator[1]);
                else
                    yield nonHeal_1.nonHeal.appendField(locator[0], value);
            }
            else
                yield nonHeal_1.nonHeal.appendField(locator, value);
        });
    }
    /**
    * Waits for an element to become not attached to the DOM on a page (by default waits for 1sec).
    * Element can be located by CSS or XPath.
    *
    * ```js
    * z.waitForDetached('#popup');
    * ```
    *
    * @param {string} locator element located by CSS|XPath|strict locator.
    * @param {number} timeout (optional, `1` by default) time in seconds to wait
    */
    static waitForDetached(locator, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator))
                yield nonHeal_1.nonHeal.waitForDetached(locator[0], timeout);
            else
                yield nonHeal_1.nonHeal.waitForDetached(locator, timeout);
        });
    }
    /**
     * Sends POST request to API.
     *
     * ```js
     * I.sendPostRequest('/api/users.json', { "email": "user@user.com" });
     * ```
     *
     * @param {*} url
     * @param {*} payload
     * @param {object} headers
     */
    static sendPostRequest(url, payload, headers) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield nonHeal_1.nonHeal.sendPostRequest(url, payload, headers);
        });
    }
    /**
     * Send GET request to REST API
     *
     * ```js
     * I.sendGetRequest('/api/users.json');
     * ```
     *
     * @param {*} url
     * @param {object} headers
     */
    static sendGetRequest(url, headers) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield nonHeal_1.nonHeal.sendGetRequest(url, headers);
        });
    }
    /**
    * Opposite to `seeElementInDOM`. Checks that element is not on page.
    *
    * ```js
    * I.dontSeeElementInDOM('.nav'); // checks that element is not on page visible or not
    * ```
    *
    * @param {string} locator located by CSS|XPath|Strict locator.
    */
    static dontSeeElementInDOM(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(locator))
                yield nonHeal_1.nonHeal.dontSeeElementInDOM(locator[0]);
            else
                yield nonHeal_1.nonHeal.dontSeeElementInDOM(locator);
        });
    }
    /**
     * Checks that text is equal to provided one.
     *
     * ```js
     * I.seeTextEquals('text', 'h1');
     * ```
     *
     * @param {string} text element value to check.
     * @param {string} optional [context]  element located by CSS|XPath|strict locator.
     */
    static seeTextEquals(text, context) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(context))
                yield nonHeal_1.nonHeal.seeTextEquals(text, context[0]);
            else
                yield nonHeal_1.nonHeal.seeTextEquals(text, context);
        });
    }
    /**
     * Checks that title is equal to provided one.
     *
     * ```js
     * z.seeTitleEquals('Test title.');
     * ```
     *
     * @param {string} text value to check.
     */
    static seeTitleEquals(text) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.seeTitleEquals(text);
        });
    }
    /**
     * Clears a cookie by name,
     * if none provided clears all cookies.
     *
     * ```js
     * z.clearCookie();
     * z.clearCookie('test');
     * ```
     *
     * @param {string} (optional, `null` by default) cookie name
     *
     */
    static clearCookie(cookie) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.clearCookie(cookie);
        });
    }
    /**
      * Retrieves the innerHTML from an element located by CSS or XPath and returns it to test.
      * Resumes test execution, so **should be used inside async function with `await`** operator.
      * If more than one element is found - an array of HTMLs returned.
      *
      * ```js
      * let postHTML = await z.grabHTMLFrom('#post');
      * ```
      *
      * @param {CodeceptJS.LocatorOrString} element located by CSS|XPath|strict locator.
      * @returns {Promise<string>} HTML code for an element
      */
    // static async grabHTMLFrom(locator:string): Promise<string>
    // {
    //     let postHTML:string;
    //     return postHTML;
    // }
    /**
     * Retrieves all the innerHTML from elements located by CSS or XPath and returns it to test.
     * Resumes test execution, so should be used inside async function with await operator.
     *
     * ```js
     * let postHTML = await z.grabHTMLFromAll('#post');
     * ```
     *
     * @param {CodeceptJS.LocatorOrString} element located by CSS|XPath|strict locator.
     * @returns {Promise<string>} HTML code for an element
     */
    static grabHTMLFromAll(locator) {
        return __awaiter(this, void 0, void 0, function* () {
            let postHTMLs = [];
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    postHTMLs = yield autoHeal_1.autoHeal.grabHTMLFromAll(locator[0], locator[1]);
                else
                    postHTMLs = yield nonHeal_1.nonHeal.grabHTMLFromAll(locator[0]);
            }
            else
                postHTMLs = yield nonHeal_1.nonHeal.grabHTMLFromAll(locator);
            return postHTMLs;
        });
    }
    /**
    * Retrieves the cssProperty from an element located by CSS or XPath and returns it to test.
    * Resumes test execution, so **should be used inside async function with `await`** operator.
    * If more than one element is found - an array of HTMLs returned.
    *
    * ```js
    * const value = await I.grabCssPropertyFrom('h3', 'font-weight');
    * ```
    *
    * @param {CodeceptJS.LocatorOrString} element located by CSS|XPath|strict locator.
    *  @param {CodeceptJS.String} cssProperty CSS property name.
    * @returns {Promise<string>} CSS value
    */
    static grabCssPropertyFrom(locator, cssProperty) {
        return __awaiter(this, void 0, void 0, function* () {
            let postHTML = '';
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    postHTML = yield autoHeal_1.autoHeal.grabCssPropertyFrom(locator[0], cssProperty, locator[1]);
                else
                    postHTML = yield nonHeal_1.nonHeal.grabCssPropertyFrom(locator[0], cssProperty);
            }
            else
                postHTML = yield nonHeal_1.nonHeal.grabCssPropertyFrom(locator, cssProperty);
            return postHTML;
        });
    }
    static grabCssPropertyFromAll(locator, cssProperty) {
        return __awaiter(this, void 0, void 0, function* () {
            let postHTML = [];
            if (Array.isArray(locator)) {
                if (process.env.AUTOHEAL == 'true' && (locator[0].startsWith("/") || locator[0].startsWith(".") || locator[0].startsWith("#") || locator[0].startsWith("{") || locator[0].startsWith("(")))
                    postHTML = yield autoHeal_1.autoHeal.grabCssPropertyFromAll(locator[0], cssProperty, locator[1]);
                else
                    postHTML = yield nonHeal_1.nonHeal.grabCssPropertyFromAll(locator[0], cssProperty);
            }
            else
                postHTML = yield nonHeal_1.nonHeal.grabCssPropertyFromAll(locator, cssProperty);
            return postHTML;
        });
    }
    static generateLog(timestamp, error) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.generateLog(timestamp, error);
        });
    }
    /**
     * Checks if element is visible and returns true of false accordingly.
     *
     * ```js
     * const value = await z.checkIfVisible("//input[@id='123']");
     * ```
     *
     * @param {string|object} locator element located by CSS|XPath|strict locator.
     * @returns {boolean} boolean
     */
    static checkIfVisible(selector, ...options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(selector))
                return yield nonHeal_1.nonHeal.checkIfVisible(selector[0], ...options);
            else
                return yield nonHeal_1.nonHeal.checkIfVisible(selector, ...options);
        });
    }
    /**
     * Downloads the file from docker agent to code base agent.
     * We can also pass an expected file name to be set for the downloaded file.
     *
     * ```js
     * const fileName = await z.dockerFileDownload();
     * const fileName = await z.dockerFileDownload("test.pdf");
     * ```
     *
     * @returns {string} file path in the repo (./tempDownload/test.pdf)
     */
    static dockerFileDownload(expectedFileName) {
        return __awaiter(this, void 0, void 0, function* () {
            yield nonHeal_1.nonHeal.dockerFileDownload(expectedFileName);
        });
    }
    /**
     * Starts the browser explicitly.
     *
     * ```js
     * await z.startBrowser();
     * ```
     */
    static startBrowser() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                console.log("========================>Start the Browser");
                yield this.prototype.helpers["WebDriver"]._startBrowser();
            }
            catch (err) {
                console.log("========================>ERROR" + err);
                console.log("========================>Starting the Browser Failed...");
            }
        });
    }
    /**
     * Stops the browser explicitly.
     *
     * ```js
     * await z.stopBrowser();
     * ```
     */
    static stopBrowser() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                console.log("========================>Stop the Browser");
                yield this.prototype.helpers["WebDriver"]._stopBrowser();
            }
            catch (err) {
                console.log("========================>ERROR" + err);
                console.log("========================>Stopping the Browser Failed...");
            }
        });
    }
}
exports.z = z;
