import { Helper } from "codeceptjs";
import { AutoHeal } from "../ElementHealer/AutoHeal";
import { GetCurrentElementInfo } from "../ElementHealer/GetCurrentElementInfo";
import { GetHealedElementInfo } from "../ElementHealer/GetHealedElementInfo";
import { GlobalVariable } from "../GlobalVariable";
const util = require('util');
import * as fs from "fs";
import * as http from "http";
const tryTo = codeceptjs.container.plugins('tryTo');
const { I } = inject();

export class autoHeal extends Helper {
    /**
    * Scrolls element into viewport, waits for that element to become visible & clickable, then performs click action.
    * Attempts auto healing of the locator
    * The second parameter is a context (CSS or XPath locator) to narrow the search.
    * ```js
    * // simple link
    * z.click('Logout');
    * // button of form
    * z.click('Submit');
    * // CSS button
    * z.click('#form input[type=submit]');
    * // XPath
    * z.click('//form/*[@type=submit]');
    * // link in context
    * z.click('Logout', '#nav');
    * // using strict locator
    * z.click({css: 'nav a.login'});
    * ```
    * @param {string} locator CSS|XPath|strict locator.
    * @param {string} context (optional, null by default) element to search in CSS|XPath|Strict locator.
    */
    static async click(locator: string, locatorName: string, timeout: number = 120, retryCount: number = 0, context?: string,) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITAITED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.click(GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar')
                                }).catch(error => {
                                    console.log('----Error in AUTOHEAL process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            console.log('-----Step Passed-----');
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.waitForClickable(locator);
                            if (util.isNullOrUndefined(context)) {
                                await I.click(locator);
                            }
                            else {
                                await I.click(locator, context);
                            }
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.waitForClickable(locator);
                            if (util.isNullOrUndefined(context)) {
                                await I.click(locator);
                            }
                            else {
                                await I.click(locator, context);
                            }
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }

                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
   * Waits for element to be visible, Clears element value & fills given value.
  * Attempts auto healing of the locator
   * ```js
   * // by label
   * z.fillField('Email', 'hello@world.com');
   * // by name
   * z.fillField('password', secret('123456'));
   * // by CSS
   * z.fillField('form#login input[name=username]', 'John');
   * // or by strict locator
   * z.fillField({css: 'form#login input[name=username]'} 'John');
   * ```
   * @param {string} locator CSS|XPath|strict locator.
   * @param {string} value
   */
    static async fillField(locator: string, value: any, locatorName: string, timeout: number = 120, retryCount: number = 0) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.fillField(GlobalVariable.HealedElement, value, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in AUTOHEAL process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.clearField(locator);
                            await I.fillField(locator, value);
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            await I.clearField(locator);
                            await I.fillField(locator, value);
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }
                    }

                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
    * Scrolls element into viewport, waits for that element to become visible & enabled, then clears field value.
   * Attempts auto healing of the locator
    * ```js
    * z.clearField('Email');
    * z.clearField('user[email]');
    * z.clearField('#email');
    * ```
    * @param {string} locator CSS|XPath|strict locator.
    */
    static async clearField(locator: string, locatorName: string, timeout: number = 120, retryCount: number = 0) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.clearField(GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.clearField(locator);
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            await I.clearField(locator);
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }

                    }

                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }



    /**
    * Checks the visibilty of an element.
    * Throws error if any of the action fails in two attempts.
    * ```js
    * z.seeElement('#modal');
    * ```
    * @param {string} locator CSS|XPath|strict locator. 
    */
    static async seeElement(locator: string, locatorName: string, timeout: number = 120, retryCount: number = 0) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForVisible(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.seeElement(GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.seeElement(locator);
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            await I.seeElement(locator);
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }

                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
  * Waits for an element to be visible. 
  * Element can be located by CSS or XPath.
  * ```js
  * z.attachFile('Avatar', 'data/avatar.jpg');
  * z.attachFile('form input[name=avatar]', 'data/avatar.jpg');
  * ```
  * @param {string} locator label|name|CSS|XPath|strict locator.
  * @param {string} filepath local file path relative to codecept.json config file
  */
    static async attachFile(locator: string, filepath: string, locatorName: string, timeout: number = 120, retryCount: number = 0) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.attachFile(GlobalVariable.HealedElement, filepath, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.attachFile(locator, filepath);
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            await I.attachFile(locator, filepath);
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }
                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }
    /**
    * Scroll element into viewport.
    * ```js
    * z.scrollIntoView('#submit');
    * z.scrollIntoView('#submit', { behavior: "smooth", block: "center", inline: "center" });
    * z.scrollIntoView('#submit', true);
    * ```
    * @param {string} locator CSS|XPath|strict locator.
    * @param {any} alignTop optional, or scrollIntoViewOptions (optional), 
    * see https://developer.mozilla.org/en-US/docs/Web/API/Element/scrollIntoView.
    
    */
    static async scrollIntoView(locator: string, alignTop?: any) {
        if (alignTop == null || alignTop == undefined) {
            await I.scrollIntoView(locator, alignTop);
        }
        else {
            await I.scrollIntoView(locator, { block: "start" })
        }
    }


    /**
    * Scrolls element into viewport, waits for that element to become visible & clickable, then performs double click action.
    * Throws error if any of the action fails in two attempts.
    * The second parameter is a context (CSS or XPath locator) to narrow the search.
    * ```js
    * z.doubleClick('Edit');
    * z.doubleClick('Edit', '.actions');
    * z.doubleClick({css: 'button.accept'});
    * z.doubleClick('.btn.edit');
    * ```
    * @param {string} locator CSS|XPath|strict locator.
    * @param {string} context (optional, null by default) element to search in CSS|XPath|Strict locator.
    */
    static async doubleClick(locator: string, locatorName: string, timeout: number = 120, retryCount: number = 0, context?: string) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.doubleClick(GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            console.log('-----Step Passed-----');
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.waitForClickable(locator);
                            if (util.isNullOrUndefined(context)) {
                                await I.doubleClick(locator);
                            }
                            else {
                                await I.doubleClick(locator, context);
                            }
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.waitForClickable(locator);
                            if (util.isNullOrUndefined(context)) {
                                await I.doubleClick(locator);
                            }
                            else {
                                await I.doubleClick(locator, context);
                            }
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }

                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
        * Scrolls element into viewport, waits for that element to become visible & enabled, then Force clicks an element
        
        * ```js
        * z.forceClick('#hiddenButton');
        * z.forceClick('Click me', '#hidden');
        * ```
       * @param {string} locator CSS|XPath|strict locator.
        * @param {string} context (optional, null by default) element to search in CSS|XPath|Strict locator.
        */
    static async forceClick(locator: string, locatorName: string, timeout: number = 120, retryCount: number = 0, context?: string) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.forceClick(GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.waitForEnabled(locator);
                            if (util.isNullOrUndefined(context)) {
                                await I.forceClick(locator);
                            }
                            else {
                                await I.forceClick(locator, context);
                            }
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.waitForEnabled(locator);
                            if (util.isNullOrUndefined(context)) {
                                await I.forceClick(locator);
                            }
                            else {
                                await I.forceClick(locator, context);
                            }
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }

                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
     * Scrolls element into viewport, waits for that element to become visible & clickable, then Drag an item to a destination element.
     * 
     * ```js
     * z.dragAndDrop('#dragHandle', '#container');
     * ```
     * @param {string} srcLocator CSS|XPath|strict locator.
     * @param {string} destLocator CSS|XPath|Strict locator.
     */
    static async dragAndDrop(srcLocator: string, destLocator: string) {
        try {
            if (srcLocator.startsWith("/") || srcLocator.startsWith(".") || srcLocator.startsWith("#") || srcLocator.startsWith("{")) {
                await I.waitForElement(srcLocator);
                await I.scrollIntoView(srcLocator, { behavior: "auto", block: "center", inline: "center" });
                await I.waitForVisible(srcLocator);
                await I.waitForClickable(srcLocator);
                await I.waitForVisible(destLocator);
                await I.dragAndDrop(srcLocator, destLocator);
            }
            else {
                await I.dragAndDrop(srcLocator, destLocator);
            }

        }
        catch (error) {
            throw new Error("dragAndDrop : Error while performing drag and drop from element located by : " + srcLocator + " to " + destLocator);
        }
    }

    /**
   * Scrolls element into viewport, waits for that element to become visible & clickable, then selects an option in a drop-down select.
   * Attempts auto healing on failure
   * 
   * ```js
   * z.selectOption('Choose Plan', 'Monthly'); // select by label
   * z.selectOption('subscription', 'Monthly'); // match option by text
   * z.selectOption('subscription', '0'); // or by value
   * z.selectOption('//form/select[@name=account]','Premium');
   * z.selectOption('form select[name=account]', 'Premium');
   * z.selectOption({css: 'form select[name=account]'} 'Premium');
   * ```
   * Provide an array for the second argument to select multiple options.
   * ```js
   * z.selectOption('Which OS do you use?', ['Android', 'iOS']);
   * ```
   * @param {string} locator CSS|XPath|strict locator.
   * @param {string|string[]} option CSS|XPath|Strict locator.
   */
    static async selectOption(locator: string, option: string, locatorName: string, timeout: number = 120, retryCount: number = 0) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.selectOption(GlobalVariable.HealedElement, option, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.waitForVisible(locator);
                            await I.waitForClickable(locator);
                            await I.selectOption(locator, option);
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.waitForVisible(locator);
                            await I.waitForClickable(locator);
                            await I.selectOption(locator, option);
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }

                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
      * Scrolls element into viewport, waits for that element to become visible & clickable. Also checks if element is already checked.
      * Then selects a checkbox or radio button.
      * 
      * The second parameter is a context (CSS or XPath locator) to narrow the search.
      * ```js
      * z.checkOption('#agree');
      * z.checkOption('I Agree to Terms and Conditions');
      * z.checkOption('agree', '//form');
      * ```
      * @param {string} locator checkbox located by label | name | CSS | XPath | strict locator.
      * @param {string} context optional element located by CSS | XPath | strict locator.
      */
    static async checkOption(locator: string, locatorName: string, timeout: number = 120, retryCount: number = 0, context?: string) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.checkOption(GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            if (!util.isNullOrUndefined(context)) {
                                await I.checkOption(locator, context);
                            }
                            else {
                                await I.checkOption(locator);
                            }
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            if (!util.isNullOrUndefined(context)) {
                                await I.checkOption(locator, context);
                            }
                            else {
                                await I.checkOption(locator);
                            }
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }

                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }


    /**
 * Scrolls element into viewport, waits for that element to become visible.
 * Then Checks that the given input field or textarea equals to given value.
 * 
 * ```js
 * z.seeInField('Username', 'davert');
 * z.seeInField({css: 'form textarea'},'Type your comment here');
 * z.seeInField('#searchform input','Search');
 * ```
 * @param {string} field located by label|name|CSS|XPath|strict locator.
 * @param {string} value value to check.
 */
    static async seeInField(locator: string, value: string, locatorName: string, timeout: number = 120, retryCount: number = 0) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.seeInField(GlobalVariable.HealedElement, value, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.waitForVisible(locator);
                            await I.seeInField(locator, value);
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.waitForVisible(locator);
                            await I.seeInField(locator, value);
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }

                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
   * Checks for element to be present, scrolls into view & Moves cursor to element matched by locator.
   * Extra shift can be set with offsetX and offsetY options.
   *
   * ```js
   * z.moveCursorTo('.tooltip');
   * z.moveCursorTo('#submit', 5,5);
   * ```
   *
   * @param {CodeceptJS.LocatorOrString} locator located by CSS|XPath|strict locator.
   * @param {number} [offsetX=0] (optional, `0` by default) X-axis offset.
   * @param {number} [offsetY=0] (optional, `0` by default) Y-axis offset.
   *
   *
   */
    static async moveCursorTo(locator: string, locatorName: string, offsetX?: number, offsetY?: number, timeout: number = 120, retryCount: number = 0) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        if (util.isNullOrUndefined(offsetX) && util.isNullOrUndefined(offsetY)) {
                                            await this.moveCursorTo(GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                        }
                                        else if (util.isNullOrUndefined(offsetX) && !(util.isNullOrUndefined(offsetY))) {
                                            await this.moveCursorTo(GlobalVariable.HealedElement, locatorName, 0, offsetY, timeout, retryCount);
                                        }
                                        else if (!(util.isNullOrUndefined(offsetX)) && util.isNullOrUndefined(offsetY)) {
                                            await this.moveCursorTo(GlobalVariable.HealedElement, locatorName, offsetX, 0, timeout, retryCount);
                                        }
                                        else {
                                            await this.moveCursorTo(GlobalVariable.HealedElement, locatorName, offsetX, offsetY, timeout, retryCount);
                                        }
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            if (util.isNullOrUndefined(offsetX) && util.isNullOrUndefined(offsetY)) {
                                await I.moveCursorTo(locator);
                            }
                            else if (util.isNullOrUndefined(offsetX) && !(util.isNullOrUndefined(offsetY))) {
                                await I.moveCursorTo(locator, 0, offsetY);
                            }
                            else if (!(util.isNullOrUndefined(offsetX)) && util.isNullOrUndefined(offsetY)) {
                                await I.moveCursorTo(locator, offsetX, 0);
                            }
                            else {
                                await I.moveCursorTo(locator, offsetX, offsetY);
                            }
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            if (util.isNullOrUndefined(offsetX) && util.isNullOrUndefined(offsetY)) {
                                await I.moveCursorTo(locator);
                            }
                            else if (util.isNullOrUndefined(offsetX) && !(util.isNullOrUndefined(offsetY))) {
                                await I.moveCursorTo(locator, 0, offsetY);
                            }
                            else if (!(util.isNullOrUndefined(offsetX)) && util.isNullOrUndefined(offsetY)) {
                                await I.moveCursorTo(locator, offsetX, 0);
                            }
                            else {
                                await I.moveCursorTo(locator, offsetX, offsetY);
                            }
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }

                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
    * Performs a swipe Right inside an element. 
    * Can be slow or fast swipe.
    * ```js
    * z.swipeRight('#container');
    *       
    * @param {string} locator an element on which to perform swipe
    * @param {string}[speed='slow'] a speed to perform: `slow` or `fast`.
    */
    static async swipeRight(locator: string, speed: string, locatorName: string, timeout: number = 120, retryCount: number = 0) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.swipeRight(GlobalVariable.HealedElement, speed, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.swipeRight(locator, speed);
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            await I.swipeRight(locator, speed);
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }

                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
* Waits for the visibility of element & for the specified value to be in value attribute.
* ```js
* z.waitForValue('//input', "GoodValue");
* ```
* @param {string} locator input field locator.
* @param {string} value
* @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
*/
    static async waitForValue(locator: string, value: string, locatorName: string, timeout: number = 120, retryCount: number = 0, context?: string) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForValue(locator, value);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.waitForValue(GlobalVariable.HealedElement, value, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }
                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
* Waits for the visibility of element.
* ```js
* z.waitForVisible('#popup');
* ```
* @param {string} locator CSS|XPath|strict locator.
* @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"  
*/
    static async waitForVisible(locator: string, locatorName: string, timeout: number = 120, retryCount: number = 0, context?: string) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForVisible(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.waitForVisible(GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {

                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }

                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
   * Waits for an element to be removed or become invisible on a page. 
   * Element can be located by CSS or XPath.
   * ```js
   * z.waitForInvisible('#popup');
   * ```
   * @param {string} locator CSS|XPath|strict locator.
   * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
   */
    static async waitForInvisible(locator: string, locatorName: string, timeout: number = 120, retryCount: number = 0, context?: string) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForInvisible(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.waitForInvisible(GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }
                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
    * Waits for an element to become enabled. 
    * Element can be located by CSS or XPath.
    * ```js
    * z.waitForEnabled('#popup');
    * ```
    * @param {string} locator CSS|XPath|strict locator.
    * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"
    */
    static async waitForEnabled(locator: string, locatorName: string, timeout: number = 120, retryCount: number = 0, context?: string) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForEnabled(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.waitForEnabled(GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }

                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
     * Waits for an element to be present on page. 
     * Element can be located by CSS or XPath.
     * ```js
     * z.waitForElement('.btn.continue');
     * z.waitForElement('.btn.continue', 5); // wait for 5 secs
     * ```
     * @param {string} locator CSS|XPath|strict locator.
     * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout" 
     */
    static async waitForElement(locator: string, locatorName: string, timeout: number = 120, retryCount: number = 0, context?: string) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.waitForElement(GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }

                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
   * Waits for an element to be clickable. 
   * Element can be located by CSS or XPath.
   * ```js
   * z.waitForClickable('.btn.continue');
   * z.waitForClickable('.btn.continue', 5); // wait for 5 secs
   * ```
   * @param {string} locator CSS|XPath|strict locator.
   * @param {number} timeout optional, default timeout is value mentioned against key "waitForTimeout"    
   */
    static async waitForClickable(locator: string, locatorName: string, timeout: number = 120, retryCount: number = 0, context?: string) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForClickable(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.waitForClickable(GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }
                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    static async grabTextFrom(locator: string, locatorName: string, timeout: number = 120, retryCount: number = 0): Promise<any> {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.grabTextFrom(GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                        resolve(GlobalVariable.text);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            GlobalVariable.text = await I.grabTextFrom(locator);
                            if (GlobalVariable.text != undefined) {
                                if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable.HealedElement = locator;
                                    await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                await I.wait(10);
                                resolve(GlobalVariable.text);
                            }
                            else
                                reject('Cannot grab text from the locator even after healing: ' + locator);
                        }
                    }
                    else {
                        if (result) {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            GlobalVariable.text = await I.grabTextFrom(locator);
                            resolve(GlobalVariable.text);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }

                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    static async grabTextFromAll(locator: string, locatorName: string, timeout: number = 120, retryCount: number = 0, context?: string): Promise<any> {
        let text: any[] = [];
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.grabTextFromAll(GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                        resolve(GlobalVariable.textFromAll);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            GlobalVariable.textFromAll = await I.grabTextFromAll(locator);
                            if (GlobalVariable.textFromAll.length != 0) {
                                if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable.HealedElement = locator;
                                    await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                await I.wait(10);
                                resolve(GlobalVariable.textFromAll);
                            }
                            else
                                reject('Cannot grab text from the locator even after healing: ' + locator);
                        }
                    }
                    else {
                        if (result) {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            GlobalVariable.textFromAll = await I.grabTextFromAll(locator);
                            resolve(GlobalVariable.textFromAll);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }
                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })

    }

    /**
        * Retrieves an attribute from an element located by CSS or XPath and returns it to test. An array as a result will be returned if there are more than one matched element. 
        * Resumes test execution, so should be used inside static async with await operator.
        * ```js
        * let hint = await z.grabAttributeFrom('#tooltip', 'title');
        * ```
        * @param {string} locator element located by CSS|XPath|strict locator.
        * @param {string} attribute attribute name.
        * @returns {string} text 
        */
    static async grabAttributeFrom(locator: string, attribute: string, locatorName: string, elementLocatorStrategy: string = 'xpath', timeout: number = 120, retryCount: number = 0): Promise<any> {
        GlobalVariable.AttributeValue = '';
        return new Promise(async (resolve, reject) => {
            GlobalVariable.text = '';
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.grabAttributeFrom(GlobalVariable.HealedElement, attribute, locatorName, elementLocatorStrategy, timeout, retryCount);
                                        resolve(GlobalVariable.AttributeValue);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.waitForVisible(locator);
                            GlobalVariable.AttributeValue = await this.grabPropertyOrAttributeValue(locator, attribute, elementLocatorStrategy);
                            if (GlobalVariable.AttributeValue != undefined) {
                                if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable.HealedElement = locator;
                                    await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                await I.wait(10);
                                resolve(GlobalVariable.AttributeValue);
                            }
                            else
                                reject('Cannot grab text from the locator even after healing: ' + locator);
                        }
                    }
                    else {
                        if (result) {
                            await I.waitForVisible(locator);
                            GlobalVariable.AttributeValue = await this.grabPropertyOrAttributeValue(locator, attribute, elementLocatorStrategy);
                            resolve(GlobalVariable.AttributeValue);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }
                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    static async grabAttributeFromAll(locator: string, attribute: string, locatorName: string, elementLocatorStrategy: string = 'xpath', timeout: number = 120, retryCount: number = 0): Promise<any> {
        GlobalVariable.AttributeValueFromAll = [];
        return new Promise(async (resolve, reject) => {
            GlobalVariable.text = '';
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.grabAttributeFromAll(GlobalVariable.HealedElement, attribute, locatorName, elementLocatorStrategy, timeout, retryCount);
                                        resolve(GlobalVariable.AttributeValueFromAll);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.waitForVisible(locator);
                            GlobalVariable.AttributeValueFromAll = await this.grabPropertyOrAttributeValueFromAll(locator, attribute, elementLocatorStrategy);
                            if (GlobalVariable.AttributeValueFromAll.length != 0) {
                                if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable.HealedElement = locator;
                                    await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                await I.wait(10);
                                resolve(GlobalVariable.AttributeValueFromAll);
                            }
                            else
                                reject('Cannot grab text from the locator even after healing: ' + locator);
                        }
                    }
                    else {
                        if (result) {
                            await I.waitForVisible(locator);
                            GlobalVariable.AttributeValueFromAll = await this.grabPropertyOrAttributeValueFromAll(locator, attribute, elementLocatorStrategy);
                            resolve(GlobalVariable.AttributeValueFromAll);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }
                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
         * Checks for element to be present, scrolls into view, waits for element to be visible and retrieves a value from a form element located by CSS or XPath, returns it to test.
         * Resumes test execution, so **should be used inside async function with `await`** operator.
         *
         * ```js
         * let email = await z.grabValueFrom('input[name=email]');
         * ```
         * @param {CodeceptJS.LocatorOrString} locator field located by label|name|CSS|XPath|strict locator.
         * @returns {Promise<any>} attribute value
         *
         */
    static async grabValueFrom(locator: string, locatorName: string, elementLocatorStrategy: string = 'xpath', timeout: number = 120, retryCount: number = 0): Promise<any> {
        GlobalVariable.AttributeValue = '';
        return new Promise(async (resolve, reject) => {
            GlobalVariable.text = '';
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.grabValueFrom(GlobalVariable.HealedElement, locatorName, elementLocatorStrategy, timeout, retryCount);
                                        resolve(GlobalVariable.AttributeValue);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.waitForVisible(locator);
                            GlobalVariable.AttributeValue = await this.grabPropertyOrAttributeValue(locator, 'value', elementLocatorStrategy);
                            if (GlobalVariable.AttributeValue != undefined) {
                                if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable.HealedElement = locator;
                                    await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                await I.wait(10);
                                resolve(GlobalVariable.AttributeValue);
                            }
                            else
                                reject('Cannot grab value from the locator even after healing: ' + locator);
                        }
                    }
                    else {
                        if (result) {
                            await I.waitForVisible(locator);
                            GlobalVariable.AttributeValue = await this.grabPropertyOrAttributeValue(locator, 'value', elementLocatorStrategy);
                            resolve(GlobalVariable.AttributeValue);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }
                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    static async grabValueFromAll(locator: string, locatorName: string, elementLocatorStrategy: string = 'xpath', timeout: number = 120, retryCount: number = 0): Promise<any> {
        GlobalVariable.AttributeValueFromAll = [];
        return new Promise(async (resolve, reject) => {
            GlobalVariable.text = '';
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.grabValueFromAll(GlobalVariable.HealedElement, locatorName, elementLocatorStrategy, timeout, retryCount);
                                        resolve(GlobalVariable.AttributeValueFromAll);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.waitForVisible(locator);
                            GlobalVariable.AttributeValueFromAll = await this.grabPropertyOrAttributeValueFromAll(locator, 'value', elementLocatorStrategy);
                            if (GlobalVariable.AttributeValueFromAll.length != 0) {
                                if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                    if (retryCount == 2)
                                        await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                    else
                                        GlobalVariable.HealedElement = locator;
                                    await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                                }
                                await I.wait(10);
                                resolve(GlobalVariable.AttributeValueFromAll);
                            }
                            else
                                reject('Cannot grab text from the locator even after healing: ' + locator);
                        }
                    }
                    else {
                        if (result) {
                            await I.waitForVisible(locator);
                            GlobalVariable.AttributeValueFromAll = await this.grabPropertyOrAttributeValueFromAll(locator, 'value', elementLocatorStrategy);
                            resolve(GlobalVariable.AttributeValueFromAll);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }
                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    static async grabPropertyOrAttributeValue(locator: string, attribute: string, elementLocatorStrategy: string = 'xpath') {
        let value = '';
        return new Promise(async (resolve, reject) => {
            try {
                const browser = await this.prototype.helpers['WebDriver'].browser;
                const elementRef = await browser.findElement(elementLocatorStrategy, locator);
                Object.keys(elementRef)
                    .forEach(async function eachKey(key) {
                        value = await browser.getElementAttribute(elementRef[key], attribute);
                        if (value == null) {
                            value = await browser.getElementProperty(elementRef[key], attribute);
                        }
                        resolve(value);
                    });
            }
            catch (error) {
                throw new Error('Error: Grab Property and attribute is throwing error for locator : ' + locator + 'while fetching attribute or property : ' + attribute);
            }
        })
    }

    static async grabPropertyOrAttributeValueFromAll(locator: string, attribute: string, elementLocatorStrategy: string = 'xpath') {
        let value: any;
        let valueArray: any = [];
        let count = 0;
        return new Promise(async (resolve, reject) => {
            try {
                const browser = await this.prototype.helpers['WebDriver'].browser;
                const elementRef = await browser.findElements(elementLocatorStrategy, locator);
                for (const element in elementRef) {
                    Object.keys(elementRef[element])
                        .forEach(async function eachKey(key) {
                            value = await browser.getElementAttribute(elementRef[element][key], attribute);
                            if (value == null) {
                                value = await browser.getElementProperty(elementRef[element][key], attribute);
                            }
                            count = count + 1;
                            valueArray.push(value);
                            if (count == elementRef.length) {
                                resolve(valueArray);
                            }
                        })

                }
            }
            catch (error) {
                reject(error);
                throw new Error('Error: Grab attribute from all is throwing error for locator : ' + locator + 'while fetching attribute or property : ' + attribute);
            }
        })
    }

    /**
         * Scrolls element into viewport, waits for that element to become visible.
         * Then Verifies that the specified checkbox is not checked.
         * 
         * ```js
         * z.dontSeeCheckboxIsChecked('#agree');
         * z.dontSeeCheckboxIsChecked('I agree to terms');
         * z.dontSeeCheckboxIsChecked('agree');
         * ```
         * @param {string} field  located by label|name|CSS|XPath|strict locator.
         */
    static async dontSeeCheckboxIsChecked(locator: string, locatorName: string, timeout: number = 120, retryCount: number = 0) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.dontSeeCheckboxIsChecked(GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.dontSeeCheckboxIsChecked(locator);
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.dontSeeCheckboxIsChecked(locator);
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }
                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
         * Scrolls element into viewport, waits for that element to become visible.
         * Then Verifies that the specified checkbox is checked.
         * 
         * ```js
         * z.seeCheckboxIsChecked('Agree');
         * z.seeCheckboxIsChecked('#agree'); 
         * z.seeCheckboxIsChecked({css: '#signup_form input[type=checkbox]'});
         * ```
         * @param {string} field  located by label|name|CSS|XPath|strict locator.
         */
    static async seeCheckboxIsChecked(locator: string, locatorName: string, timeout: number = 120, retryCount: number = 0) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.seeCheckboxIsChecked(GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.seeCheckboxIsChecked(locator);
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.seeCheckboxIsChecked(locator);
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }
                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
         * Checks that all elements with given locator have given attributes.
         *
         * ```js
         * z.seeAttributesOnElements('//form', { method: "post"});
         * ```
         *
         * @param {CodeceptJS.LocatorOrString} locator located by CSS|XPath|strict locator.
         * @param {object} attributes attributes and their values to check.
         *
         */
    static async seeAttributesOnElements(locator: string, attributes: any, locatorName: string, timeout: number = 120, retryCount: number = 0, context?: string) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.seeAttributesOnElements(GlobalVariable.HealedElement, attributes, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.seeAttributesOnElements(locator, attributes);
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            await I.seeAttributesOnElements(locator, attributes);
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }
                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
        * Appends text into the field.
        * A field can be located by text, accessibility id, id.
        *
        * ```js
        * z.appendField('name', 'davert');
        * ```
        *
        * @param {string} field
        * @param {string} value
        */
    static async appendField(locator: string, value: string, locatorName: string, timeout: number = 120, retryCount: number = 0, context?: string) {
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.appendField(GlobalVariable.HealedElement, value, locatorName, timeout, retryCount);
                                        resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.click(locator);
                            await I.appendField(locator, value);
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(true);
                        }
                    }
                    else {
                        if (result) {
                            await I.click(locator);
                            await I.appendField(locator, value);
                            resolve(true);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }
                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
             * Retrieves the innerHTML from an element located by CSS or XPath and returns it to test.
             * Resumes test execution, so **should be used inside async function with `await`** operator.
             * If more than one element is found - an array of HTMLs returned.
             *
             * ```js
             * let postHTML = await z.grabHTMLFrom('#post');
             * ```
             *
             * @param {CodeceptJS.LocatorOrString} element located by CSS|XPath|strict locator.
             * @returns {Promise<string>} HTML code for an element
             */
    static async grabHTMLFrom(locator: string, locatorName: string, timeout: number = 120, retryCount: number = 0, context?: string) {
        GlobalVariable.postHTML = '';
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.grabHTMLFrom(GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                        resolve(GlobalVariable.postHTML); resolve(true);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.waitForVisible(locator);
                            GlobalVariable.postHTML = await I.grabHTMLFrom(locator);
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(GlobalVariable.postHTML);
                        }
                    }
                    else {
                        if (result) {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.waitForVisible(locator);
                            resolve(GlobalVariable.postHTML);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }
                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
             * Retrieves all the innerHTML from elements located by CSS or XPath and returns it to test. 
             * Resumes test execution, so should be used inside async function with await operator.
             *
             * ```js
             * let postHTML = await z.grabHTMLFromAll('#post');
             * ```
             *
             * @param {CodeceptJS.LocatorOrString} element located by CSS|XPath|strict locator.
             * @returns {Promise<string>} HTML code for an element
             */
    static async grabHTMLFromAll(locator: string, locatorName: string, timeout: number = 120, retryCount: number = 0, context?: string): Promise<any> {
        GlobalVariable.postHTMLFromAll = '';
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.grabHTMLFromAll(GlobalVariable.HealedElement, locatorName, timeout, retryCount);
                                        resolve(GlobalVariable.postHTMLFromAll);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.waitForVisible(locator);
                            GlobalVariable.postHTMLFromAll = await I.grabHTMLFromAll(locator);
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(GlobalVariable.postHTMLFromAll);
                        }
                    }
                    else {
                        if (result) {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.waitForVisible(locator);
                            GlobalVariable.postHTMLFromAll = await I.grabHTMLFromAll(locator);
                            resolve(GlobalVariable.postHTMLFromAll);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }
                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    /**
            * Retrieves the cssProperty from an element located by CSS or XPath and returns it to test.
            * Resumes test execution, so **should be used inside async function with `await`** operator.
            * If more than one element is found - an array of HTMLs returned.
            *
            * ```js
            * const value = await I.grabCssPropertyFrom('h3', 'font-weight');
            * ```
            *
            * @param {CodeceptJS.LocatorOrString} element located by CSS|XPath|strict locator.
            *  @param {CodeceptJS.String} cssProperty CSS property name.
            * @returns {Promise<string>} CSS value
            */
    static async grabCssPropertyFrom(locator: string, cssProperty: string, locatorName: string, timeout: number = 120, retryCount: number = 0, context?: string): Promise<string> {
        GlobalVariable.postHTML = '';
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {
                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.grabCssPropertyFrom(GlobalVariable.HealedElement, cssProperty, locatorName, timeout, retryCount);
                                        resolve(GlobalVariable.postHTML);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.waitForVisible(locator);
                            GlobalVariable.postHTML = await I.grabCssPropertyFrom(locator, cssProperty);
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(GlobalVariable.postHTML);
                        }
                    }
                    else {
                        if (result) {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.waitForVisible(locator);
                            GlobalVariable.postHTML = await I.grabCssPropertyFrom(locator, cssProperty);
                            resolve(GlobalVariable.postHTML);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }
                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }

    static async grabCssPropertyFromAll(locator: string, cssProperty: string, locatorName: string, timeout: number = 120, retryCount: number = 0, context?: string): Promise<any> {
        GlobalVariable.postHTMLFromAll = '';
        return new Promise(async (resolve, reject) => {
            if(retryCount == 0)
            GlobalVariable.OriginalXpath=locator;
            if (retryCount <= 1) {
                await tryTo(async () => {
                    try {
                        await I.waitForElement(locator, timeout);
                        GlobalVariable.error = 'NA'
                    }
                    catch (error) {
                        console.log('Error with element : ' + error);
                        GlobalVariable.error = error;
                    }
                }).then(async (result: boolean) => {
                    if (!util.isNullOrUndefined(locatorName)) {
                        await GetCurrentElementInfo.storeInfoInJson(locatorName, retryCount);
                        if ((GlobalVariable.ElementMap.get('autoUpdate') == 0 || retryCount == 1) && result) {
                            if (retryCount != 1)
                                GlobalVariable.HealedElement = locator;
                            await GetHealedElementInfo.getElementInfo(GlobalVariable.HealedElement);
                        }
                        retryCount += 1;
                        if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') != 0) {
                            if (((GlobalVariable.error).toString()).includes('still not present') || ((GlobalVariable.error).toString()).includes('still not visible')) {
                                console.log('-----AUTOHEAL INITIATED-----');
                                GlobalVariable.ElementMap.set('autoUpdate', 0);
                                await AutoHeal.startHealing(locatorName).then(async () => {

                                    if (GlobalVariable.HealedElement != 'NA') {
                                        await this.grabCssPropertyFromAll(GlobalVariable.HealedElement, cssProperty, locatorName, timeout, retryCount);
                                        resolve(GlobalVariable.postHTMLFromAll);
                                    }
                                    else
                                        reject('Healed xpath does not meet the minimum confidence score bar');
                                }).catch(error => {
                                    console.log('----Error in healing process----');
                                    reject(error);
                                })
                            }
                            else {
                                console.log('----Step failed due to : ' + GlobalVariable.error);
                                reject(GlobalVariable.error);
                            }
                        }
                        else if (!result && retryCount > 1) {

                            reject('Failed on healing >>>>  Old Xpath : ' + locator + ' Healed Xpath used : ' + GlobalVariable.HealedElement);
                        }
                        else if (!result && retryCount == 1 && GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                            console.log('*************Healing NA**************');
                            reject('Test Step Failed and no healing was performed as element info was not present ')
                        }
                        else {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.waitForVisible(locator);
                            GlobalVariable.postHTMLFromAll = await I.grabCssPropertyFromAll(locator, cssProperty);
                            if (retryCount == 2 || GlobalVariable.ElementMap.get('autoUpdate') == 0) {
                                if (retryCount == 2)
                                    await GetHealedElementInfo.replaceHealedXpath(locatorName);
                                else
                                    GlobalVariable.HealedElement = locator;
                                await GetCurrentElementInfo.updateDetailsInJson(locatorName);
                            }
                            await I.wait(10);
                            resolve(GlobalVariable.postHTMLFromAll);
                        }
                    }
                    else {
                        if (result) {
                            await I.scrollIntoView(locator, { behavior: "auto", block: "center", inline: "center" });
                            await I.waitForVisible(locator);
                            GlobalVariable.postHTMLFromAll = await I.grabCssPropertyFromAll(locator, cssProperty);
                            resolve(GlobalVariable.postHTMLFromAll);
                        }
                        else {
                            reject('-----Test Step Failed and no AUTOHEAL adopted-----')
                        }
                    }
                }).catch((error: any) => {
                    reject(error);
                })
            }
        })
    }
}