import { GlobalVariable } from "../GlobalVariable";
import fs from 'fs';
import { GetCurrentElementInfo } from "./GetCurrentElementInfo";

/* Get attributes and properties of the healed test object from from the DOM*/
const { I } = inject();
export class GetHealedElementInfo extends Helper {
    static async getElementInfo(locator: string) {
        GlobalVariable.ElementMap.set('className', await this.getElementClassName(locator) as string);
        GlobalVariable.ElementMap.set('tag', await this.getElementTag(locator) as string);
        GlobalVariable.ElementMap.set('id', await this.getElementId(locator) as string);
        GlobalVariable.ElementMap.set('attribute', await this.getElementAttributes(locator) as string);
        GlobalVariable.ElementMap.set('parent', await this.getElementParent(locator) as string);
        console.log('elem map -- ' + [...GlobalVariable.ElementMap]);
    }

    static async getElementClassName(locator: string) {
        let value = '';
        return new Promise(async (resolve) => {
            try {
                const browser = await this.prototype.helpers['WebDriver'].browser;
                const elementRef = await browser.findElement('xpath', locator);
                console.log('element ref class= ' + JSON.stringify(elementRef));
                Object.keys(elementRef)
                    .forEach(async function eachKey(key) {
                        if (key == 'error') {
                            console.log('-------Error in element ----');
                            resolve('NA')
                        }
                        value = await browser.getElementProperty(elementRef[key], 'className');
                        resolve(value);
                    });
            }
            catch (error) {
                resolve('NA')
            }
        })
    }

    static async getElementTag(locator: string) {
        let value = '';
        return new Promise(async (resolve) => {
            try {
                const browser = await this.prototype.helpers['WebDriver'].browser;
                const elementRef = await browser.findElement('xpath', locator);
                Object.keys(elementRef)
                    .forEach(async function eachKey(key) {
                        if (key == 'error') {
                            console.log('-------Error in element ----');
                            resolve('NA')
                        }
                        value = await browser.getElementProperty(elementRef[key], 'tagName');
                        resolve(value);
                    });
            }
            catch (error) {
                resolve('NA')
            }
        })
    }

    static async getElementId(locator: string) {
        let value = '';
        return new Promise(async (resolve) => {
            try {
                const browser = await this.prototype.helpers['WebDriver'].browser;
                const elementRef = await browser.findElement('xpath', locator);
                Object.keys(elementRef)
                    .forEach(async function eachKey(key) {
                        if (key == 'error') {
                            console.log('-------Error in element ----');
                            resolve('NA')
                        }
                        value = await browser.getElementProperty(elementRef[key], 'id');
                        resolve(value);
                    });
            }
            catch (error) {
                resolve('NA')
            }
        })
    }

    static async getElementSize(locator: string) {
        let value = '';
        return new Promise(async (resolve) => {
            try {
                const browser = await this.prototype.helpers['WebDriver'].browser;
                const elementRef = await browser.findElement('xpath', locator);
                Object.keys(elementRef)
                    .forEach(async function eachKey(key) {
                        if (key == 'error') {
                            console.log('-------Error in element ----');
                            resolve({})
                        }
                        value = await elementRef.getSize('width');
                        resolve(value);
                    });
            }
            catch (error) {
                resolve({})
            }
        })
    }

    static async getElementParent(locator: string) {
        let value = '';
        return new Promise(async (resolve) => {
            try {
                const browser = await this.prototype.helpers['WebDriver'].browser;
                const elementRef = await browser.findElement('xpath', locator);
                Object.keys(elementRef)
                    .forEach(async function eachKey(key) {
                        if (key == 'error') {
                            console.log('-------Error in element ----');
                            resolve('NA')
                        }
                        value = await browser.getElementProperty(elementRef[key], 'parentElement');
                        let parentTag = await browser.getElementProperty(Object.values(value)[0], 'tagName');
                        resolve(parentTag);
                    });
            }
            catch (error) {
                resolve('NA')
            }
        })
    }

    static async getElementAttributes(locator: string) {
        let value: any;
        let attributeArr: any = {};
        return new Promise(async (resolve) => {
            try {
                const browser = await this.prototype.helpers['WebDriver'].browser;
                const elementRef = await browser.findElement('xpath', locator);
                Object.keys(elementRef)
                    .forEach(async function eachKey(key) {
                        if (key == 'error') {
                            console.log('-------Error in element ----');
                            resolve('NA')
                        }
                        value = await browser.getElementProperty(elementRef[key], 'attributes');
                        for (let j = 0; j < value.length; j += 1) {
                            attributeArr[value[j]['nodeName']] = value[j]['nodeValue'];
                        }
                        resolve(attributeArr);
                    });
            }
            catch (error) {
                resolve(attributeArr)
            }
        })
    }

    static async storeElementInfoInElementFile(locator: string, locatorName: string) {
        let obj = {
            "CLASS_NAME": GlobalVariable.ElementMap.get('className'),
            "TAG": GlobalVariable.ElementMap.get('tag'),
            "ID": GlobalVariable.ElementMap.get('id'),
            "SIZE": GlobalVariable.ElementMap.get('size'),
            "PARENT": GlobalVariable.ElementMap.get('parent')
        }

    }

    static async replaceHealedXpath(locatorName: string) {
        let k = 0;
        for (let i = 0; i < GlobalVariable.autoHealReport.length; i += 1) {
            if ((GlobalVariable.autoHealReport)[i]['LocatorName'] == locatorName) {
                console.log('Element Alredy healed');
                k = 1;
            }
        }
        if (k == 0) {
            console.log('**** Step : Replace Healed xpath ****' + GlobalVariable.HealedElement);
            const allurePlugin = codeceptjs.container.plugins('allure');
            await allurePlugin.createStep('****AUTOHEALING DONE FOR ' + GlobalVariable.OriginalXpath + '****  Healed Xpath: ' + GlobalVariable.HealedElement, () => { })
            await this.writeToJson(locatorName);
        }
    }

    static async objInitialization(locatorName: string) {
        try {
            let obj: any = {};
            obj['LocatorName'] = locatorName;
            obj['Old Xpath'] = GlobalVariable.OriginalXpath;
            obj['Healed Xpath'] = GlobalVariable.HealedElement;
            return obj;
        }
        catch (error) {
            console.log(error + 'in object initialization');
        }
    }

    static async readAutoHealJson(): Promise<any> {
        return new Promise((resolve, reject) => {
            try {
                resolve(JSON.parse(fs.readFileSync('./autoHealReport.json', "utf-8")))
            } catch (error) {
                console.log(error + ' in reading json file');
                reject(error)
            }
        })

    }

    static async writeToJson(locatorName: string) {
        console.log("started writing to FA json");
        await Promise.all([GlobalVariable.autoHealReport, this.objInitialization(locatorName)]).then((allPromises) => {
            try {
                JSON.parse(JSON.stringify(allPromises[1]));
                try {
                    allPromises[0].push(allPromises[1]);
                    JSON.parse(JSON.stringify(allPromises[0], null, "\t"));
                    try {
                        fs.writeFileSync('./autoHealReport.json', JSON.stringify(allPromises[0], null, "\t"));
                    }
                    catch (error) {
                        console.log(error + 'in writing to file');
                    }
                }
                catch (error) {
                    console.log('Error in parsing combined promise = ' + error);
                }
            }
            catch (error) {
                console.log('Error in parsing promise 1 =  ' + error);
            }
        }).catch(error => {
            console.log(error + 'in fulfilling promise');
        });
    }
}